package pse.client.framework;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;

import java.io.*;
import java.util.*;

import pse.client.*;
import pse.shared.PSEException;

import com.sun.midp.io.InternalConnector;

public class DeviceClient extends MIDlet implements UIListener, DeviceMessengerListener, ProfileHandlerFinder {

  /**
   * Variabili che descrivono lo stato di interazione del client con il framework
   */
  private boolean loadedWrappers = false;
  private boolean searchingPSE = false;
  private boolean foundPSE = false;
  private boolean loggedIn = false;

  /**
   * Gli schermi che sono utilizzati dal client
   */
  WorkingScreen workingScreen; //sempre la stessa istanza
  PromptScreen promptScreen; //creato e distrutto quando serve
  StepsScreen stepsScreen; //creato e distrutto quando serve
  InitialScreen initialScreen; //sempre la stessa istanza
  OfflineWrapperInfoScreen offlineWrapperInfoScreen; //creato e distrutto quando serve
  ChoosingFrameworkScreen choosingFrameworkScreen; //sempre la stessa
  LoginScreen loginScreen; //creato e distrutto quando serve
  MainScreen mainScreen; //sempre la stessa

  /**
   * Variabile di stato per mantenere la situazione corrente di visualizzazione
   * e relativi valori ammissibili
   */
  int displayState;
  static final int PROMPT_SCREEN = 1;
  static final int INITIAL_SCREEN = 2;
  static final int CHOOSING_SCREEN = 3;
  static final int LOGIN_SCREEN = 4;
  static final int MAIN_SCREEN = 5;
  static final int MENU_SCREEN = 6;

   /**
    * Varibili utili per il tarttamento dell'interfaccia grafica
    */
  int performingAction;
  Thread performingThread;
  static final int LOADING_WRAPPERS = 1;
  static final int SEARCHING_FRAMEWORK = 2;
  static final int LOGGING_IN = 3;
  static final int LOGGING_OUT = 4;
  static final int FINDING_PROXIES = 5;
  static final int STARTING_SERVICE = 6;
  static final int STOPPING_SERVICE = 7;
  static final int PAUSING_SERVICE = 8;
  static final int RESUMING_SERVICE = 9;

  /**
   * Varibile che mantiene il nome del wrapper attualmente accreditato all'uso della UI
   */
  private String visualizedWrapper = null;
   //Variabili ancora da mettere a posto

   /**
   * Mantiene i nomi dei servizi in esecuzione
   */
  private Vector executingServices = new Vector();

  //Variabili aggiunte in tesi03
  //Socket s1, s2;
  StreamConnection s1, s2;
  DeviceMessenger messenger;

  //Variabili per il surrogate
  private final int PROTOCOL_VERSION = 1;
  //private final String multicastRequestAddressName = "224.0.1.95";
  private final int multicastRequestPort = 1925;
  private final String jarFileName = "surrogate.jar";

  //private InetAddress multicastRequestAddress;
  private final String multicastRequestAddress = "224.0.1.95";
  //private InetAddress myAddress;
  private String myAddress;
  private int unicastResponsePort;

  //private DatagramSocket reqRespSocket;
  private DatagramConnection reqRespSocket;
  //Provo a usare quella sopra sia per mandare che per ricevere
  //private DatagramSocket respSocket;
  //private InetAddress registrationAddress;
  private String registrationAddress;
  private int registrationTCPPort;
  private int registrationUDPPort;

  private boolean accepting = false, accepted = false; //TODO: da sostituire con qualcosa di pi consistente per la sincronizzazione

  private MonitoringAlivementThread monitoringAlivementThread;

  private static final String installedServicesFile = "installed.txt";
  //questo dovrebbe essere qui per il problema di midp con indirizzi ip numerici. Evidentemente risolverlo poi in altro modo
  private static String surrogateAddress = "localhost";

  /**
   * Hashtable che mantiene le corrispondenze (Nome del servizio - ServiceDescriptor).
   */
  private Hashtable wrappersHT = new Hashtable();

  //public MIDlet mySelf;

  /**Construct the midlet*/
  public DeviceClient() {
  }

  /**Main method*/
  public void startApp() {
    Displayable current = Display.getDisplay(this).getCurrent();
    if (current == null) {
      promptScreen = new PromptScreen(this);
      workingScreen = new WorkingScreen(this);
      choosingFrameworkScreen = new ChoosingFrameworkScreen(this); //istanziato qui per motivi di tempistica, ma visto che si usa sempre la stessa istanza non ci sono problemi
      Display.getDisplay(this).setCurrent(promptScreen);
      displayState = PROMPT_SCREEN;
    }
    else {
      Display.getDisplay(this).setCurrent(current);
    }
  }

  /**Handle pausing the MIDlet*/
  public void pauseApp() {
    //TODO: Dovrei fremare i thread? E se non lo faccio?
  }

  /**Handle destroying the MIDlet*/
  public void destroyApp(boolean unconditional) {
    monitoringAlivementThread = null;
    if (messenger != null) messenger.destroy();
    if (s1!=null) {
      try {
        s1.close();
      }
      catch (IOException ex) {
      }
    }
    if (s2!=null) {
      try {
        s2.close();
      }
      catch (IOException ex) {
      }
    }
  }

  /**Quit the MIDlet*/
  public void quitApp() {
    destroyApp(true);
    notifyDestroyed();
  }

  /**
   * Metodi di ProfileHandlerFinder
   */
  public void notifyFinished(String wName) {
    ServiceDescriptor sDesc = (ServiceDescriptor)wrappersHT.get(wName);
    if (sDesc == null) {
      return;
    }
    int selState = sDesc.getState();
    if (!((selState==ServiceDescriptor.PAUSED)||(selState==ServiceDescriptor.RUNNING))) {
      //showStatusMessage("The wrapper is not executing.  Can't stop it");
      System.out.println("The wrapper is not executing.  Can't stop it");
      return;
    }
    int brIndex = sDesc.getBridgeIndex();
    try {
      messenger.stopBridge(brIndex);
    }
    catch (PSEException ex) {
      //showStatusMessage("Error in stopping Bridge: "+ex.getMessage()+" Execution continues");
      System.out.println("Error in stopping Bridge: "+ex.getMessage()+" Execution continues");
    }
    try {
      messenger.stoppedService(brIndex);
    }
    catch (PSEException ex) {
      //showStatusMessage("Error in notifying service stopping : "+ex.getMessage()+" Execution continues");
      System.out.println("Error in notifying service stopping : "+ex.getMessage()+" Execution continues");
    }
    sDesc.setState(ServiceDescriptor.SEARCH_SUCCEEDED);
    executingServices.removeElement(wName);
    //TODO modificare cos anche il prec
  }

  public boolean showScreen(Displayable disp, String wrapperName) {
    if (visualizedWrapper == null) return false;
    if (visualizedWrapper.equals(wrapperName)) {
      Display.getDisplay(this).setCurrent(disp);
      return true;
    }
    return false;
  }

  public Display getDisplay() {
    return Display.getDisplay(this);
  }

  public void selectedUIServiceBack() {
    //System.out.println("selectUIBack - da implementare");
    visualizedWrapper = null;
    Display.getDisplay(this).setCurrent(mainScreen);
  }

  /**
   * Metodi il log di informazioni, il trattamento di errori e la transizione di schermi da WorkingScreen
   */
  private void log(String msg) {
    workingScreen.appendMessage(msg+"\n");
  }

  private void operationOK() {
    if (displayState == PROMPT_SCREEN) {
      if (performingAction == LOADING_WRAPPERS) {
        System.out.println("Il caricamento dei wrappers  andato bene. Dovrei ora impostare lo schermo successivo");
        if (initialScreen == null) {
          initialScreen = new InitialScreen(this, wrappersHT.keys());
        }
        Display.getDisplay(this).setCurrent(initialScreen);
        displayState = INITIAL_SCREEN;
      }
    }
    else if (displayState == INITIAL_SCREEN) {
      if (performingAction == SEARCHING_FRAMEWORK) {
        System.out.println("La ricerca del Framework  andata bene. Dovrei ora passare allo schermo successivo");
        //choosingFrameworkScreen = new ChoosingFrameworkScreen(this);
        Display.getDisplay(this).setCurrent(choosingFrameworkScreen);
        displayState = CHOOSING_SCREEN;
      }
    }
    else if (displayState == LOGIN_SCREEN) {
      if (performingAction == LOGGING_IN) {
        System.out.println("Il login  andato bene. Dovrei ora passare allo schermo successivo");
        if (mainScreen == null) {
          mainScreen = new MainScreen(this, wrappersHT.keys());
        }
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
    }
    else if (displayState == MAIN_SCREEN) {
      if (performingAction == LOGGING_OUT) {
        System.out.println("Il logout  andato bene. Dovrei ora passare allo schermo successivo");
        loginScreen = new LoginScreen(this);
        Display.getDisplay(this).setCurrent(loginScreen);
        displayState = LOGIN_SCREEN;
      }
    }
    else if (displayState == MENU_SCREEN) {
      if (performingAction == FINDING_PROXIES) {
        System.out.println("Il ritrovamento dei proxies  andato bene. Dovrei passare al menu successivo. Per ora torno semplvemente al Main");
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == STARTING_SERVICE) {
        System.out.println("Il servizio  stato attivato correttamente. Dovrebbe entrare in gioco l'interfaccia dello schermo");
        //imposto un PuppetServiceScreen che potr poi essere rimpiazzato da quello proprio del servizio
        PuppetServiceScreen puppetScreen = new PuppetServiceScreen(this);
        Display.getDisplay(this).setCurrent(puppetScreen);
        //No il servizio dovr implementare la sua interfaccia graifca con back
      }
      else if (performingAction == STOPPING_SERVICE) {
        System.out.println("Il servizio  stato fermato correttamente. Ritorno al mainScreen");
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == PAUSING_SERVICE) {
        System.out.println("Il servizio  stato messo in pausa correttamente. Ritorno al mainScreen");
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == RESUMING_SERVICE) {
        System.out.println("Il servizio  stato riattivato correttamente. Ritorno al mainScreen");
        //imposto un PuppetServiceScreen che potr poi essere rimpiazzato da quello proprio del servizio
        PuppetServiceScreen puppetScreen = new PuppetServiceScreen(this);
        Display.getDisplay(this).setCurrent(puppetScreen);
      }
    }
  }

  private void operationFail(String failureMsg) {
    if (displayState == PROMPT_SCREEN) {
      if (performingAction == LOADING_WRAPPERS) {
        log(failureMsg);
        Alert alert = new Alert("Error in loading Wrappers", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
        alert = null;
      }
    }
    else if (displayState == INITIAL_SCREEN) {
      if (performingAction == SEARCHING_FRAMEWORK) {
        log(failureMsg);
        Alert alert = new Alert("Error in searching PSE Framework", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
        alert = null;
      }
    }
    else if (displayState == LOGIN_SCREEN) {
      if (performingAction == LOGGING_IN) {
        log(failureMsg);
        Alert alert = new Alert("Error in logging in", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
        alert = null;
      }
    }
    else if (displayState == MAIN_SCREEN) {
      if (performingAction == LOGGING_OUT) {
        log(failureMsg);
        Alert alert = new Alert("Error in logging out", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
        alert = null;
      }
    }
    else if (displayState == MENU_SCREEN) {
      if (performingAction == FINDING_PROXIES) {
        log(failureMsg);
        Alert alert = new Alert("Error in finding proxies", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
        alert = null;
      }
      else if (performingAction == STARTING_SERVICE) {
        log(failureMsg);
        Alert alert = new Alert("Error in starting service", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
      }
      else if (performingAction == STOPPING_SERVICE) {
        log(failureMsg);
        Alert alert = new Alert("Error in stopping service", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
      }
      else if (performingAction == PAUSING_SERVICE) {
        log(failureMsg);
        Alert alert = new Alert("Error in pausing service", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
      }
      else if (performingAction == RESUMING_SERVICE) {
        log(failureMsg);
        Alert alert = new Alert("Error in resuming service", failureMsg, null, AlertType.ERROR);
        alert.setTimeout(5000);
        stepsScreen = new StepsScreen(this, workingScreen.getTitle(), workingScreen.getString());
        Display.getDisplay(this).setCurrent(alert, stepsScreen);
        System.out.println("Tornato da alert");
      }
    }
  }

  public void doBackFromSteps() {
    if (displayState == PROMPT_SCREEN) {
      if (performingAction == LOADING_WRAPPERS) {
        promptScreen = new PromptScreen(this);
        Display.getDisplay(this).setCurrent(promptScreen);
        stepsScreen = null;
      }
    }
    else if (displayState == INITIAL_SCREEN) {
      if (performingAction == SEARCHING_FRAMEWORK) {
        Display.getDisplay(this).setCurrent(initialScreen);
        stepsScreen = null;
      }
    }
    else if (displayState == LOGIN_SCREEN) {
      if (performingAction == LOGGING_IN) {
        Display.getDisplay(this).setCurrent(loginScreen);
        stepsScreen = null;
      }
    }
    else if (displayState == MAIN_SCREEN) {
      if (performingAction == LOGGING_OUT) {
        //Anche se il logout va male, cmq torno allo schermo precedente
        loginScreen = new LoginScreen(this);
        Display.getDisplay(this).setCurrent(loginScreen);
        displayState = LOGIN_SCREEN;
      }
    }
    else if (displayState == MENU_SCREEN) {
      if (performingAction == FINDING_PROXIES) {
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == STARTING_SERVICE) {
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == STOPPING_SERVICE) {
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == PAUSING_SERVICE) {
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
      else if (performingAction == RESUMING_SERVICE) {
        Display.getDisplay(this).setCurrent(mainScreen);
        displayState = MAIN_SCREEN;
      }
    }
  }

  /**
   * Risposta agli eventi in PromptScreen
   */
  public void doExitAtPrompt() {
    quitApp();
  }

  public void doStart() {
    //Ora dovrei caricare i wrappers e testare la soluzione con schermo intermedio
    workingScreen.setMessage("Loading wrappers ...");
    performingAction = LOADING_WRAPPERS;
    Display.getDisplay(this).setCurrent(workingScreen);
    promptScreen = null;
    performingThread = new Thread() {
        public void run() {
          findWrappers();
        }
    };
    performingThread.start();
    //Imposto il nuovo schermo e poi
    //promptScreen = null;
  }

  /**
   * Metodi ..
   */

  private void findWrappers() /*throws PSEException*/ {
    //Leggi il file installedServicesFile e carica le classi indicate per ottenere informazioni
    InputStream is = null;
    try {
      is = getClass().getResourceAsStream("/"+installedServicesFile);
    }
    catch (Exception ex) {
      //throw new PSEException("Unable to open file "+installedServicesFile+" for reading Wrappers' classes\n"+ex.getMessage());
      operationFail("Unable to open file "+installedServicesFile+" for reading Wrappers' classes\n"+ex.getMessage());
      return;
    }
    if (is == null) {
      operationFail("Unable to open file "+installedServicesFile+" for reading Wrappers' classes");
      return;
    }
    log("File "+installedServicesFile+" opened. Reading wrappers classes ...");
    String line = null;
    wrappersHT = new Hashtable();
    Vector wrapperNames = new Vector();
    Hashtable wrapperIndexes = new Hashtable();
    Vector serviceInterfaces = new Vector();
    Hashtable interfacesIndex = new Hashtable();
    int bridgeIndex = 0;
    try {
      line = readLine(is);
    }
    catch (IOException ex) {
      //throw new PSEException("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
      operationFail("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
      return;
    }
    while ((line!=null) && (line.length()>0)) {
      System.out.println("Ho letto : "+line);
      Class cl = null;
      ServiceWrapper sw = null;
      try {
        //System.out.println("loading -1");
        //cl = Class.forName("pse.temp.client.Wrapper");
        //System.out.println("Faccio compare = "+line.compareTo("pse.temp.client.Wrapper"));
        cl = Class.forName(line);
        //System.out.println("loading -2");
        sw = (ServiceWrapper)cl.newInstance();
        //System.out.println("loading -3");
      }
      catch (Exception ex) {
        //throw new PSEException("Error in loading Wrapper class: "+ex.getMessage());
        log("Error in loading Wrapper class: "+ex.getMessage());
      }
      System.out.println("Ho trovato : "+sw.getWrapperInfo());
      log("Found: "+sw.getWrapperInfo());
      String proposedName = sw.getWrapperInfo().getServiceName();
      int proposedIndex = 1;
      //Faccio in modo che due servizi non abbiano lo stesso nome
      if (wrapperNames.contains(proposedName)) {
        String key = (String)wrapperNames.elementAt(wrapperNames.indexOf(proposedName));
        Integer oldIndex = (Integer)wrapperIndexes.get(key);
        Integer newIndex = new Integer(oldIndex.intValue()+1);
        wrapperIndexes.remove(key);
        wrapperIndexes.put(key, newIndex);
        proposedName = proposedName+"("+newIndex.intValue()+")";
      }
      else {
        wrapperNames.addElement(proposedName);
        wrapperIndexes.put(proposedName, new Integer(1));
      }
      //Ordino ed assegno un indice unico ai wrapper che fanno riferimento alla stessa interfaccia di servizio
      String servInt = sw.getWrapperInfo().getServiceInterface();
      if (serviceInterfaces.contains(servInt)) {
        String key = (String)serviceInterfaces.elementAt(serviceInterfaces.indexOf(servInt));
        Integer oldIndex = (Integer)interfacesIndex.get(key);
        Integer newIndex = new Integer(oldIndex.intValue()+1);
        interfacesIndex.remove(key);
        interfacesIndex.put(key, newIndex);
        proposedIndex = newIndex.intValue();
      }
      else {
        serviceInterfaces.addElement(servInt);
        interfacesIndex.put(servInt, new Integer(1));
      }
      ServiceDescriptor sd = new ServiceDescriptor(sw);
      sd.setWrapperIndex(proposedIndex);
      sd.setBridgeIndex(bridgeIndex++);
      wrappersHT.put(proposedName, sd);
      try {
        line = readLine(is);
      }
      catch (IOException ex) {
        //throw new PSEException("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
        operationFail("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
        return;
      }
    }
    loadedWrappers = true;
    operationOK();
  }

  /**
   * Legge una riga dall'inputeStream.
   * Come BufferedReader.readLine(). Solo che in MIDP la classe BufferedReader non  disponibile
   */
  private String readLine(InputStream input) throws IOException {
    boolean eol = false;
    int current;
    StringBuffer buf = new StringBuffer();
    while (!eol) {
      current = input.read();
      //System.out.println("Ho letto "+current);
      if (current == 13) {
        input.read();
        eol = true;
      }
      else if (current == -1) eol = true;
      else {
        buf.append((char)current);
      }
    }
    return new String(buf);
  }

  /**
   * Risposta ai metodi di InitialScreen
   */
  public void doExitAtInitial() {
    System.out.println("doExitAtInitial() - da implementare");
    //Magari dire al surrogate che  finita
    quitApp();
  }

  public void doViewWrapperInfo(String wrapperName) {
    System.out.println("doViewWrapperInfo("+wrapperName+") - da implementare");
    //Non c' bisogno di transizione verbosa. Tratto tutto qui dentro
    ServiceDescriptor sd = (ServiceDescriptor)wrappersHT.get(wrapperName);
    ServiceWrapper sw = sd.getServiceWrapper();
    String wn = sw.getWrapperInfo().getServiceName();
    String si = sw.getWrapperInfo().getServiceInterface();
    WrapperProfileJ2ME pr = sw.getWrapperProfile();
    offlineWrapperInfoScreen = new OfflineWrapperInfoScreen(this, this, wn, si, pr);
    Display.getDisplay(this).setCurrent(offlineWrapperInfoScreen);
  }

  public void doSearchFramework() {
    //Se  la prima volta che richiedo di ricercare il framework ...
    if (!searchingPSE) {
      workingScreen.setMessage("Searching Framework ...");
      performingAction = SEARCHING_FRAMEWORK;
      Display.getDisplay(this).setCurrent(workingScreen);
      performingThread = new Thread() {
          public void run() {
            searchPSE();
          }
      };
      performingThread.start();
    }
    //altrimenti visualizza i framework offerti finora, senza ricercare il surrogate host, ecc
    else {
      Display.getDisplay(this).setCurrent(choosingFrameworkScreen);
    }
  }

  /**
   * Metodi richiamati nella risposta in InitialScreen
   */

  public void searchPSE() {
    //Qua devo avviare tuttalaproceduradel surrogate:
    try {
      findSurrogateHost();
    }
    catch (PSEException ex) {
      operationFail("Error in finding surrogate host: "+ex.getMessage());
      return;
    }
    log("Surrogate host found.\nRegistering the surrogate...");
    try {
      registerSurrogate();
    }
    catch (PSEException ex) {
      operationFail("Error in registering surrogate: "+ex.getMessage());
      return;
    }
    log("Surrogate registered. Waiting for establishing connection with it ...");
    System.out.println("Aspetto che le socket siano state aperte");
    while (!accepted) {
      try {
        Thread.sleep(500);
      }
      catch (Exception ex) {
      }
    }
    System.out.println("Aposto: s1 e s2 sono state aperte");
    messenger = new DeviceMessenger(this, s1, s2);
    //Invoco searchPSE tramite di esso
    //System.out.println("searchPSE - 1");
    log("Connection established.\nSearching for PSE Framework in the network ...");
    searchingPSE = true;
    try {
      messenger.findFramework();
    }
    catch (PSEException ex) {
      operationFail("Error in finding Framework");
      searchingPSE = false;
      return;
    }
    log("Waiting for Framework offers.");
    operationOK();
  }

  private void findSurrogateHost() throws PSEException {
    try {
      myAddress = getMyAddress();
    } catch (Exception e) {
        System.out.println("Errore nella lettura del mio indirizzo: "+e.getMessage());
        throw new PSEException("Errore nella lettura del mio indirizzo: "+e.getMessage());
      }
    try {
      //reqRespSocket = new DatagramSocket(0);
      //TODO Qua poi invece che 45674 sarebbe meglio fare un ciclo...
      reqRespSocket = (DatagramConnection)InternalConnector.openInternal("datagram://:45674", Connector.READ_WRITE, false);
      //unicastResponsePort =reqRespSocket.getLocalPort();
      unicastResponsePort = 45674;
      //System.out.println("Socket registrata: Indirizzo = "+reqRespSocket.getLocalAddress().getHostAddress());
      //System.out.println("Porta = "+unicastResponsePort);
    } catch (Exception e) {
        System.out.println("Errore nella creazione della socket reqSocket :"+e.getMessage());
        if (reqRespSocket!=null) {
          try {
            reqRespSocket.close();
          }
          catch (Exception ex) {
          }
        }
        throw new PSEException("Errore nella creazione della socket reqSocket :"+e.getMessage());
    }
    try {
      //Mando la richiesta
      System.out.println("Costruisco il datagram packet...");
      //DatagramPacket pkt = marshalOutgoingHostRequest(multicastRequestAddress, multicastRequestPort, unicastResponsePort);
      Datagram pkt = marshalOutgoingHostRequest(multicastRequestAddress, multicastRequestPort, unicastResponsePort);
      System.out.println("ho costruito il datagramma e vale" + pkt.toString());
      System.out.println("Invio il datagramma all'indirizzo di Multicast");
      reqRespSocket.send(pkt);
      System.out.println("Mandata la richiesta. Ora attendo la risposta...");
      log("Multicast Request sent. Waiting for response ...");
      byte[] buf = new byte[512];
      //DatagramPacket rpkt = new DatagramPacket(buf, buf.length);
      Datagram rpkt = reqRespSocket.newDatagram(buf, buf.length);
      reqRespSocket.receive(rpkt);
      System.out.println("Ho ricevuto qualcosa. Adesso parso...");
      DataInputStream dis =null;
      try {
        byte[] bytes = rpkt.getData();
        dis = new DataInputStream(new ByteArrayInputStream(bytes));
        IncomingHostResponse resp = new IncomingHostResponse(dis);
        registrationAddress = resp.getHostAddress();
        registrationTCPPort = resp.tcpPort;
        registrationUDPPort=  resp.udpPort;
        System.out.println("Il messaggio che ho ricevuto e' cosi' composto : \nAddress = "+resp.getHostAddress()+"\nTCP port = "+resp.tcpPort+"\nUDP port = "+resp.udpPort);
      } catch (Exception e) {
        System.out.println("Errore in lettura della risposte: "+e.getMessage());
        throw new PSEException("Errore in lettura della risposte: "+e.getMessage());
      }
    } catch (Exception e) {
        System.out.println("Errore in findSurrogateHost() "+e.getMessage());
        throw new PSEException("Errore in findSurrogateHost() "+e.getMessage());
      }
    try {
      reqRespSocket.close();
    }
    catch (Exception ex) {
    }
  }//findSurrogateHost

  private void registerSurrogate() throws PSEException {
    System.out.println("registerSurrogate()");
    //Socket sock = null;
    StreamConnection sock = null;
    //ServerSocket ss = null;
    StreamConnectionNotifier ss = null;
    DataOutputStream dos = null;
    try {
      System.out.println("Apro la socket per registrare il surrogate");
      //sock = new Socket(registrationAddress, registrationTCPPort);
      sock = (StreamConnection)InternalConnector.openInternal("socket://"+registrationAddress+":"+registrationTCPPort, Connector.READ_WRITE, false);
      System.out.println("Ho aperto la connessione di registrazione su : "+"socket://"+registrationAddress+":"+registrationTCPPort);
      //ss = new ServerSocket(0);
      //TODO anche qui come per le datagram-server manca metodo per poi reperire la porta, eventualmente fare ciclo
      ss = (StreamConnectionNotifier)InternalConnector.openInternal("serversocket://:7645", Connector.READ_WRITE, false);
      new ListeningThread(ss).start();
      while (!accepting) {
        try {
          //Pre per mettere alla socket di mettersi in ascolto...
          System.out.println("Dormo un po'");
          Thread.sleep(500);
          System.out.println("Ora mi sveglio");
        }
        catch (Exception ex) {
        }
      }
      // generate initialization data
      System.out.println("Costruisco l'initialization data");
      System.gc();
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      dos = new DataOutputStream(bos);
      //dos.writeShort(ss.getLocalPort());
      //NEW Anche se anche in questo caso ci vorrebbe qualcosa di meglio che una porta statica
      dos.writeShort(7645);
      //System.out.println("Ho scritto :"+ss.getLocalPort());
      //NEW
      System.out.println("Ho scritto :"+7645);
      dos.flush();
      byte[] initData = bos.toByteArray();
      //NEW
      /*
      dos.close();
      bos.close();
      bos = null;
      dos = null;
      */
      //dos = new DataOutputStream(sock.getOutputStream());
      dos = sock.openDataOutputStream();
      dos.writeInt(PROTOCOL_VERSION);
      System.out.println("Scrivo: "+PROTOCOL_VERSION);

      /*

          SUrrogate URL. Prima no


      dos.writeShort(0); // set the length of surrogate URL as 0
      System.out.println("Scrivo: 0");
      */

      //Se ci metto 192.168.1.1 non va (d errore 'No routing')
      String surrURL="http://localhost:8082/surrogate.jar";
      byte [] bSurrURL = surrURL.getBytes();
      dos.writeShort(bSurrURL.length);
      System.out.println("Scrivo: "+bSurrURL.length);
      dos.write(bSurrURL, 0, bSurrURL.length);
      System.out.println("Scrivo: "+bSurrURL.toString());
      dos.writeInt(initData.length);
      System.out.println("Scrivo: "+initData.length);
      dos.write(initData, 0, initData.length);
      System.out.println("Scrivo: il contenuto di initData");
      dos.writeInt(0); // set the length of surrogate as 0
      System.out.println("Scrivo: 0");
      dos.flush();

      // write the jar out
      //dos.writeInt((int)jarFile.length());
      //System.out.println("Scrivo: "+(int)jarFile.length());
      //NEW Bisogna che prima scandisca tutto il jar:
      /*
            SCRITTURA DEL SURROGATE DAL JAR. DA' PROBLEMI DI HEAP

      System.out.println("Predo il contenuto del file jar");
      //File jarFile = new File(jarFileName);
      //FileInputStream fis = new FileInputStream(jarFile);
      System.out.println("Filtro...");
      Class cl = getClass();
      System.out.println("HO recuperato la class");
      InputStream fis = cl.getResourceAsStream("/"+jarFileName);
      //InputStream fis = getClass().getResourceAsStream("/"+jarFileName);
      System.out.println("Ho trovato "+jarFileName+" ? "+fis );
      System.out.println("Scrivo il messaggio sulla socket...");
      System.out.println("Conto da quanti bytes  costituito il jar...");
      byte[] buff = new byte[1024];
      int count = 0;
      int len = fis.read(buff);
      while (len > 0) {
          //dos.write(buff, 0, len);
          count+=len;
          len = fis.read(buff);
      }
      dos.writeInt(count);
      System.out.println("Scrivo: "+count);
      //NEW Apro e richiudo. Ho finito di contare: questa volta scrivo proprio sulla socket
      fis.close();
      fis = getClass().getResourceAsStream("/"+jarFileName);
      len = fis.read(buff);
      while (len > 0) {
          dos.write(buff, 0, len);
          //System.out.println("Ne scrivo : "+len);
          len = fis.read(buff);
      }
      dos.flush();
      System.out.println("Scrivo: il contenuto del jar file");
      */
    }
    catch (Exception ex) {
      System.out.println("Errore nella fase ri registrazione del surrogate: "+ex.getMessage());
      try {
          if (dos != null)
            dos.close();
          if (sock != null)
              sock.close();
          /*if (ss != null)
            ss.close();*/
      } catch(IOException e) {}
      throw new PSEException("Errore nella fase ri registrazione del surrogate: "+ex.getMessage());
    }
    //ATT.NE! Questo  fondamentale! Altrimenti madison lancia socketException (ma perch poi ?!?!?)
    try {
        if (dos != null)
          dos.close();
        if (sock != null)
            sock.close();
        /*if (ss != null)
          ss.close();*/
    } catch(IOException e) {}
  }//registerSurrogate


  private String getMyAddress() {
    //Versione temporanea. Poi bisogner prevedere in altro modo, per. es. tramite il server MidpIpAddress
    return "me";
  }

  /**
   * Costruisce un Datagram di Multicast Host Request
   * @param hostAddres indirizzo multicast a cui mandare la richiesta
   * @param porta a cui mandare la richiesta
   * @param porta su cui si attendono risposte
   */
  //private DatagramPacket marshalOutgoingHostRequest(InetAddress hostAddr, int reqPort, int respPort) throws IOException {
  //NEW
  private Datagram marshalOutgoingHostRequest(String hostAddr, int reqPort, int respPort) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    DataOutputStream dos = new DataOutputStream(bos);
    dos.writeInt(PROTOCOL_VERSION);
    //String addr = deviceAddress.getHostAddress();
    String addr = myAddress;
    dos.writeShort(addr.length());
    //dos.writeBytes(addr);
    //NEW
    dos.write(addr.getBytes());
    dos.writeShort(respPort);
    dos.flush();
    byte[] message = bos.toByteArray();
    /*if (message.length > Constants.MAX_MULTICAST_PKT_SIZE)
        throw new IllegalArgumentException("device address marshals " +
                                           "too large");*/
    //return new Datagram(message, message.length, hostAddr, reqPort);
    //NEW
    return reqRespSocket.newDatagram(message, message.length, "datagram://"+hostAddr+":"+reqPort);
  }

  private class IncomingHostResponse {
    //private InetAddress hostAddress;
    //NEW
    private String hostAddress;
    private int tcpPort;
    private int udpPort;

    /** Parse a response message from a host. */
    IncomingHostResponse(DataInputStream dis) throws IOException {
      int version = dis.readInt();
      if (version != PROTOCOL_VERSION)
        throw new IOException("unsupported protocol version: " + version);
      // host address
      int addrLen = dis.readUnsignedShort();
      if (addrLen > 0) {
        byte[] addrData = new byte[addrLen];
        dis.readFully(addrData);
        //hostAddress = InetAddress.getByName(new String(addrData));
        //NEW
        hostAddress = new String(addrData);
      } else {
          /*if (System.getProperty("com.sun.jini.madison.debug") != null)
              System.err.println("Host response with no host address!");*/
          throw new IOException("Host response with no host address!");
      }
      tcpPort = dis.readUnsignedShort();
      udpPort = dis.readUnsignedShort();
      dis.close();
    }

    /*public InetAddress getHostAddress() {
        return hostAddress;
    }*/
    //NEW
    public String getHostAddress() {
      return hostAddress;
    }

    public int getTCPPort() {
      return tcpPort;
    }

    public int getUDPPort() {
      return udpPort;
    }

    /*public boolean equals(Object o) {
        if (!(o instanceof IncomingHostResponse))
            return false;
        IncomingHostResponse resp = (IncomingHostResponse)o;
        return hostAddress.equals(resp.hostAddress) &&
            tcpPort == resp.tcpPort && udpPort == resp.udpPort;
    }*/
  }//IncomingHostResponse

  private class ListeningThread extends Thread {
    //ServerSocket servSock;
    StreamConnectionNotifier servSock;

    ListeningThread(StreamConnectionNotifier servSock) {
      this.servSock = servSock;
    }

    public void run() {
      try {
        accepting =true;
        System.out.println("Sono ListeningSocket. Sto per fare accept()");
        //s1 = servSock.accept();
        s1 = servSock.acceptAndOpen();
        System.out.println("Sono ListeningSocket. Ho fatto la prima accept()");
        //s2 = servSock.accept();
        s2 = servSock.acceptAndOpen();
        System.out.println("Sono ListeningSocket. Ho fttto la seconda accept()");
        accepted= true;
        servSock.close();
      } catch (Exception e) {
          System.out.println("Errore nella accept: "+e.getMessage());
      } finally {
          try {
            servSock.close();
          }
          catch (Exception ex) {
            System.out.println("Errore nella chiusura di ss");
          }
      }
    }
  }//ListeningThread

   /**
   * Metodo per il keepAlive. Deve rispondere con il messaggio di KeepAliveSuccess ed
   * eventualmente prevedere un thread per controlare che entro il long arrivi un'altra richiesta di
   * keep alive (per ora questo non viene fatto)
   */
  public void keepAlive(long period) {
    System.out.println("Invocato Client01.keepAlive("+period+")");
    try {
      messenger.deviceAlive();
    }
    catch (Exception ex) {
      System.out.println("Error in signalling alivement to surrogate");
    }
    if (monitoringAlivementThread == null) {
      monitoringAlivementThread = new MonitoringAlivementThread(System.currentTimeMillis()+period+10000);
      monitoringAlivementThread.start();
    }
    else {
      monitoringAlivementThread.setNextCheck(System.currentTimeMillis()+period+10000); /*10 secondi di margine*/
    }
  }

  class MonitoringAlivementThread extends Thread {
    Object sem = new Object();
    long nextCheck;
    MonitoringAlivementThread(long nextCheckMillis) {
      synchronized (sem) {
        nextCheck = nextCheckMillis;
      }
    }
    public void setNextCheck(long nextCheckMillis) {
      synchronized (sem) {
        nextCheck = nextCheckMillis;
      }
    }
    public void run () {
      long currentTime = System.currentTimeMillis();
      Thread currentThread = Thread.currentThread();
      while (currentThread == monitoringAlivementThread) {
        currentTime = System.currentTimeMillis();
        long tempNextCheck = nextCheck;
        System.out.println("Itero: currentTime = "+currentTime+" tempNextCheck = "+tempNextCheck);
        if (tempNextCheck>currentTime) {
          try {
            Thread.sleep(tempNextCheck-currentTime);
          }
          catch (InterruptedException ex) {
          }
        }
        else {
          System.out.println("Non rilevato in tempo messaggio di alivement");
          //TODO fare qlcs di pi?
          monitoringAlivementThread = null;
        }
      }
    }
  }//MonitoringAlivementThread

  public void foundFramework() {
    //foundPSE = true;
    //searchingPSE = false;
    System.out.println("Invocato foundFramework");
    if (searchingPSE) {
      choosingFrameworkScreen.addFramework("framework1");
    }
  }

  /**
   * Risposta ai metodi di OfflineWrapperInfoScreen
   */
  public void doBackFromOfflineWrapperInfo() {
    Display.getDisplay(this).setCurrent(initialScreen);
    offlineWrapperInfoScreen = null;
  }

  /**
   * Risposta ai metodi di ChoosingFrameworkScreen
   */
  public void doChoseFramework() { //Poi qua bisogner passare indicativo
    System.out.println("doChoseFramework - Ancora da implementare");
    //Imposta Schermo di Login
    loginScreen = new LoginScreen(this);
    Display.getDisplay(this).setCurrent(loginScreen);
    displayState = LOGIN_SCREEN;
    foundPSE = true;
    //searchingPSE = false;
  }

  public void doBackFromChoosingFramework() {
    //TODO Qua in realt bisognerebbe anche fermare tutte le cose nella resistrazione in corso (
    //s1, s2, listeningThread, l'alivemeent, insomma chidere la sessione corrente con il surrogate host,
    //oppure fare in modo di non ricercare l'host, ma solo attendere per nuove offerte di
    Display.getDisplay(this).setCurrent(initialScreen);
    //searchingPSE = false;
  }

  public void doViewFrameworkInfo() {
    System.out.println("doViewFrameworkInfo - Ancora da implementare");
  }

  /**
   * Risposta ai metodi di LoginScreen
   */
  public void doLogin(final String userName, final String pwd) {
    workingScreen.setMessage("Logging in ...");
    performingAction = LOGGING_IN;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          loginToPSE(userName, pwd);
        }
    };
    performingThread.start();
  }

  public void doBackFromLogin() {
    Display.getDisplay(this).setCurrent(choosingFrameworkScreen);
    displayState = CHOOSING_SCREEN;
    loginScreen = null;
  }

  private void loginToPSE(String un, String pwd) {
    System.out.println("Richiesto Login con "+un+"-"+pwd);
    if (!loadedWrappers) { //In realt questa verifica  inutile
      operationFail("Wrappers not loaded yet");
      return;
    }
    if (!foundPSE) {
      operationFail("Framework not found yet");
      return;
    }
    if (loggedIn) {
      operationFail("Session already active. Please logout before login again");
      return;
    }
    log("Sending request to surrogate ...");
    try {
      messenger.login(un, pwd, new DeviceProfileJ2ME());
    }
    catch (PSEException ex) {
      operationFail("Error in login: "+ex.getMessage());
      return;
    }
    loggedIn = true;
    operationOK();
    //loggedIn();
  }

  /**
   * Risposta ai metodi di MainScreen
   */
  public void doLogout() {
    System.out.println("doLogout - non implementato");
    workingScreen.setMessage("Logging out ...");
    performingAction = LOGGING_OUT;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          logoutFromPSE();
        }
    };
    performingThread.start();
  }

  public void doHandleService(String wrapperName) {
    System.out.println("doHandleService - non implementato");
    //Non c' bisogno di transizione verbosa. Tratto tutto qui dentro
    ServiceDescriptor sd = (ServiceDescriptor)wrappersHT.get(wrapperName);
    ServiceWrapper sw = sd.getServiceWrapper();
    //String wn = sw.getWrapperInfo().getServiceName();
    /*
    String si = sw.getWrapperInfo().getServiceInterface();
    WrapperProfileJ2ME pr = sw.getWrapperProfile();
    */
    int state = sd.getState();
    Screen sc = null;
    FoundServicesProfiles fsp;
    int pi;
    switch (state) {
      case ServiceDescriptor.NOT_SEARCHED:
      case ServiceDescriptor.SEARCH_FAILED:
        //sc = new AloneWrapperInfoScreen(this, this, wn, si, pr, state);
        sc = new AloneWrapperMenuScreen(this, wrapperName);
        break;
      case ServiceDescriptor.SEARCH_SUCCEEDED:
        fsp = sd.getFoundServicesProfiles();
        //sc = new IdleWrapperInfoScreen(this, this, wn, si, pr, state, fsp);
        sc = new SearchSuccededWrapperMenuScreen(this, this, wrapperName, fsp);
        break;
      case ServiceDescriptor.RUNNING:
        fsp = sd.getFoundServicesProfiles();
        pi = sd.getActiveProxyIndex();
        sc = new RunningWrapperMenuScreen(this, this, wrapperName, fsp);
        break;
      case ServiceDescriptor.PAUSED:
        fsp = sd.getFoundServicesProfiles();
        pi = sd.getActiveProxyIndex();
        sc = new PausedWrapperMenuScreen(this, this, wrapperName, fsp);
        break;
      default:
        //c' qlcs che non va...
    }
    Display.getDisplay(this).setCurrent(sc);
    displayState=MENU_SCREEN;
    //offlineWrapperInfoScreen = new OfflineWrapperInfoScreen(this, this, wn, si, pr);
    //Display.getDisplay(this).setCurrent(offlineWrapperInfoScreen);
  }

  public void doChangeProfile(String wrapperName) {
    System.out.println("doChangeProfile - non implementato");
  }

  public void doViewActiveWrapperInfo(String wrapperName) {
    System.out.println("doViewActiveWrapperInfo("+wrapperName+") - da implementare");
    //Non c' bisogno di transizione verbosa. Tratto tutto qui dentro
    ServiceDescriptor sd = (ServiceDescriptor)wrappersHT.get(wrapperName);
    ServiceWrapper sw = sd.getServiceWrapper();
    //String wn = sw.getWrapperInfo().getServiceName();
    String si = sw.getWrapperInfo().getServiceInterface();
    WrapperProfileJ2ME pr = sw.getWrapperProfile();
    int state = sd.getState();
    System.out.println("doWiew... state = "+state);
    Screen sc = null;
    FoundServicesProfiles fsp;
    int pi;
    switch (state) {
      case ServiceDescriptor.NOT_SEARCHED:
      case ServiceDescriptor.SEARCH_FAILED:
        sc = new AloneWrapperInfoScreen(this, this, wrapperName, si, pr, state);
        break;
      case ServiceDescriptor.SEARCH_SUCCEEDED:
        fsp = sd.getFoundServicesProfiles();
        sc = new IdleWrapperInfoScreen(this, this, wrapperName, si, pr, state, fsp);
        break;
      case ServiceDescriptor.RUNNING:
      case ServiceDescriptor.PAUSED:
        fsp = sd.getFoundServicesProfiles();
        pi = sd.getActiveProxyIndex();
        sc = new ExecutingWrapperInfoScreen(this, this, wrapperName, si, pr, state, fsp, pi);
        break;
      default:
        //c' qlcs che non va...
    }
    Display.getDisplay(this).setCurrent(sc);
    //offlineWrapperInfoScreen = new OfflineWrapperInfoScreen(this, this, wn, si, pr);
    //Display.getDisplay(this).setCurrent(offlineWrapperInfoScreen);
  }

  private void logoutFromPSE() {
    //System.out.println("logoutButton_actionPerformed");
    if (!loggedIn) {
      operationFail("Framework not found yet");
      loggedIn = false;
      return;
    }
    if (!executingServices.isEmpty()) {
      operationFail("There are services running. Stop them before logging out");
      loggedIn = false;
      return;
    }
    try {
      messenger.logout();
    }
    catch (PSEException ex) {
      operationFail("Error in logout: "+ex.getMessage());
      loggedIn = false;
      return;
    }
    loggedIn = false;
    operationOK();
  }

  /**
   * Risposta ai metodi comuni ai WrapperInfoScreen
   */
  public void doBackFromWrapperInfo() {
    //System.out.println("doBackFromWrapperInfo - non implementato");
    Display.getDisplay(this).setCurrent(mainScreen);
  }

  public void doUseService(String wrapperName) {
    //System.out.println("doUseService - non implementato");
    doHandleService(wrapperName);
  }

  /**
   * Metodi comuni a tutti i WrapperMenu
   */
  public void doBackFromWrapperMenu() {
    //System.out.println("doBackFromWrapperMenu - non implementato");
    Display.getDisplay(this).setCurrent(mainScreen);
    displayState = MAIN_SCREEN;
  }

  /**
   * Metodi da AloneWrapperMenuScreen
   */
  public void doFindProxies(final String wrapperName) {
    System.out.println("doFindProxies - non implementato");
    workingScreen.setMessage("Finding proxies for "+wrapperName+" ...");
    performingAction = FINDING_PROXIES;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          findProxies(wrapperName);
        }
    };
    performingThread.start();
  }

  void findProxies(String wrapperName) {
    if (!loggedIn) {
      operationFail("No Session active. Please login before");
      return;
    }
    ServiceDescriptor selDesc = (ServiceDescriptor)wrappersHT.get(wrapperName);
    if (selDesc == null) {
      operationFail("No descriptor associated with this wrapper");
      return;
    }
    int selState = selDesc.getState();
    if ((selState == ServiceDescriptor.RUNNING) || (selState == ServiceDescriptor.PAUSED)) {
      operationFail("The wrapper is executing. Exit it before searching for new proxies");
      return;
    }
    ServiceWrapper sw = selDesc.getServiceWrapper();
    if (sw==null) {
      operationFail("Wrapper not found!");
      return;
    }
    int wInd = selDesc.getWrapperIndex();
    FoundServicesProfiles foundServicesProfiles = null;
    try {
      /* DOPO DISTINGUERE I DUE CASI
      if (selState == ServiceDescriptor.SEARCH_SUCCEEDED) {
        foundServicesProfiles = messenger.findService(selDesc.getBridgeIndex(),
          sw.getWrapperInfo().getServiceInterface(), wInd, );
      }
      else {
        foundServices = pseInterface.findService(sessionID, sw.getWrapperInfo().getServiceInterface(), wInd, sw.getWrapperProfile());
      }*/
      if (selState == ServiceDescriptor.SEARCH_SUCCEEDED) {
        log("Sending request ...");
        foundServicesProfiles = messenger.findServices(selDesc.getBridgeIndex());
      }
      else {
        log("Sending request for the first time ...");
        foundServicesProfiles = messenger.findServices(selDesc.getBridgeIndex(),
            sw.getWrapperInfo().getServiceInterface(), wInd, sw.getWrapperProfile(), sw.getBridgeName());
      }
    }
    catch (PSEException ex) {
      operationFail("Error in choosing Proxy : "+ex.getMessage());
      selDesc.setState(ServiceDescriptor.SEARCH_FAILED);
      return;
    }
    if (foundServicesProfiles == null) {
      operationFail("Error in choosing Proxy : Empty");
      selDesc.setState(ServiceDescriptor.SEARCH_FAILED);
      return;
    }
    System.out.println("Sono startService(). Sono stati scaricati i proxies. Ora se ce n'e' piu' d'uno dovrei sceglierlo e poi iniziare l'elaborazione");
    System.out.println("La descrizione dei servizi trovati e': "+foundServicesProfiles.toString());
    if (foundServicesProfiles.getNumberOfServices() == 0) {
      operationFail("No matching proxy found");
      selDesc.setState(ServiceDescriptor.SEARCH_FAILED);
      return;
    }
    selDesc.setState(ServiceDescriptor.SEARCH_SUCCEEDED);
    selDesc.setFoundServicesProfiles(foundServicesProfiles);
    //CANCEL
    //Aggiornamento della visualzzazione dei proxy
    operationOK();
  }

  /**
   * Metodi di SearchSucceededWrapperMenuScreen
   */
  public void doStartService(final String wrapperName, final int proxyIndex) {
    System.out.println("doStartService - non implementato");
    workingScreen.setMessage("Starting service "+wrapperName+":"+proxyIndex+" ...");
    performingAction = STARTING_SERVICE;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          startService(wrapperName, proxyIndex);
        }
    };
    performingThread.start();
  }

  void startService(String wrapperName, int proxyIndex) {
    //Fare tutti i controlli su null e sugli stati ammissibili in cui posso essere, e poi anche:
    if (!loggedIn) {
      operationFail("No Session active. Please login before");
      return;
    }
    ServiceDescriptor sDesc = (ServiceDescriptor)wrappersHT.get(wrapperName);
    if (sDesc == null) {
      operationFail("No descritpion associated with this wrapper");
      return;
    }
    int selState = sDesc.getState();
    if (selState!=ServiceDescriptor.SEARCH_SUCCEEDED) {
      operationFail("The wrapper has not searched proxies yet.  (Not in state search succeded)");
      return;
    }
    ServiceWrapper sw = sDesc.getServiceWrapper();
    if (sw == null) {
      operationFail("No wrapper present");
      return;
    }
    int wrIndex = sDesc.getWrapperIndex();
    int brIndex = sDesc.getBridgeIndex();
    int bridgePort;
    try {
      bridgePort = messenger.startBridge(brIndex, proxyIndex);
    }
    catch (PSEException ex) {
      operationFail("Error in starting Bridge: "+ex.getMessage());
      try {
        messenger.stopBridge(brIndex);
      }
      catch (PSEException exc) {}
      return;
    }
    System.out.println("Ho avviato il brige; la porta a cui aspetta : "+bridgePort);
    try {
      sw.start(this, wrapperName, surrogateAddress, bridgePort);
    }
    catch (PSEException ex) {
      operationFail("Error in starting wrapper "+ex.getMessage());
      sw.stop();
      try {
        messenger.stopBridge(brIndex);
      }
      catch (Exception exc) {
      }
      return;
    }
    try {
      messenger.startedService(brIndex);
    }
    catch (PSEException ex) {
      operationFail("Error in notifying service starting : "+ex.getMessage()+" Execution continues");
    }
    executingServices.addElement(wrapperName);
    sDesc.setState(ServiceDescriptor.RUNNING);
    sDesc.setActiveProxyIndex(proxyIndex);
    operationOK();
    visualizedWrapper = wrapperName;
    sw.run();
  }

  /**
   * Metodi da RunningWrapperMenuScreen
   */
  public void doPauseService(final String wrapperName) {
    System.out.println("doPauseService - non implementato");
    workingScreen.setMessage("Pausing service "+wrapperName);
    performingAction = PAUSING_SERVICE;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          pauseService(wrapperName);
        }
    };
    performingThread.start();
  }

  private void pauseService(String wName) {
    //Fare tutti i controlli su null e sugli stati ammissibili in cui posso essere
    if (!loggedIn) {
      operationFail("No Session active. Please login before");
      return;
    }
    ServiceDescriptor sDesc = (ServiceDescriptor)wrappersHT.get(wName);
    if (sDesc == null) {
      operationFail("No descritpion associated with this wrapper");
      return;
    }
    int selState = sDesc.getState();
    if (selState!=ServiceDescriptor.RUNNING) {
      operationFail("The wrapper is not running");
    }
    ServiceWrapper sw = sDesc.getServiceWrapper();
    if (sw == null) {
      operationFail("No wrapper present");
      return;
    }
    sw.pause();
    //pseInterface.stoppedService();
    sDesc.setState(ServiceDescriptor.PAUSED);
    visualizedWrapper = null;
    operationOK();
  }

  public void doStopService(final String wrapperName) {
    System.out.println("doStopService - non implementato");
    workingScreen.setMessage("Stopping service "+wrapperName);
    performingAction = STOPPING_SERVICE;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          stopService(wrapperName);
        }
    };
    performingThread.start();
  }

  void stopService(String wrapperName) {
    //Fare tutti i controlli su null e sugli stati ammissibili in cui posso essere
    if (!loggedIn) {
      operationFail("No Session active. Please login before");
      return;
    }
    ServiceDescriptor sDesc = (ServiceDescriptor)wrappersHT.get(wrapperName);
    if (sDesc == null) {
      operationFail("No descritpion associated with this wrapper");
      return;
    }
    int selState = sDesc.getState();
    if (!((selState==ServiceDescriptor.PAUSED)||(selState==ServiceDescriptor.RUNNING))) {
      operationFail("The wrapper is not executing.  Can't stop it");
    }
    ServiceWrapper sw = sDesc.getServiceWrapper();
    if (sw == null) {
      operationFail("No wrapper present");
      return;
    }
    //Faccio come fanno coi MIDLET, cio se sono in pausa prima riavvio e poi stoppo
    if (sDesc.getState()==ServiceDescriptor.PAUSED) sw.resume();
    sw.stop();
    int brIndex = sDesc.getBridgeIndex();
    int bridgePort;
    try {
      messenger.stopBridge(brIndex);
    }
    catch (PSEException ex) {
      operationFail("Error in stopping Bridge: "+ex.getMessage()+" Execution continues");
    }
    try {
      messenger.stoppedService(brIndex);
    }
    catch (PSEException ex) {
      operationFail("Error in notifying service stopping : "+ex.getMessage()+" Execution continues");
    }
    executingServices.removeElement(wrapperName);
    sDesc.setState(ServiceDescriptor.SEARCH_SUCCEEDED);
    //In realt dovrebbe gi essere null, cmq...
    visualizedWrapper = null;
    operationOK();
  }

  /**
   * Metodi da PausedWrapperMenuScreen
   */
  public void doResumeService(final String wrapperName) {
    System.out.println("doResumeService - non implementato");
    workingScreen.setMessage("Resuming service "+wrapperName);
    performingAction = RESUMING_SERVICE;
    Display.getDisplay(this).setCurrent(workingScreen);
    performingThread = new Thread() {
        public void run() {
          resumeService(wrapperName);
        }
    };
    performingThread.start();
  }

  private void resumeService(String wName) {
    //Fare tutti i controlli su null e sugli stati ammissibili in cui posso essere
    if (!loggedIn) {
      operationFail("No Session active. Please login before");
      return;
    }
    ServiceDescriptor sDesc = (ServiceDescriptor)wrappersHT.get(wName);
    if (sDesc == null) {
      operationFail("No descritpion associated with this wrapper");
      return;
    }
    int selState = sDesc.getState();
    if (selState!=ServiceDescriptor.PAUSED) {
      operationFail("The wrapper is not in pause");
    }
    ServiceWrapper sw = sDesc.getServiceWrapper();
    if (sw == null) {
      operationFail("No wrapper present");
      return;
    }
    visualizedWrapper = wName;
    sw.resume();
    sDesc.setState(ServiceDescriptor.RUNNING);
    operationOK();
  }
}