package pse.client.framework;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;

import java.io.*;
import java.util.*;

import pse.client.*;
import pse.shared.PSEException;

import com.sun.midp.io.InternalConnector;

/**
 * Per fare alcuni tentativi iniziali e per testare innanzitutto se riesco
 * a caricare i wrappers, poi a caricare il surrogate
 */
public class MIDlet1 extends MIDlet implements DeviceMessengerListener, UIListener {

  //Variabili aggiunte in tesi03
  //Socket s1, s2;
  StreamConnection s1, s2;
  DeviceMessenger messenger;

  //Variabili per il surrogate
  private final int PROTOCOL_VERSION = 1;
  //private final String multicastRequestAddressName = "224.0.1.95";
  private final int multicastRequestPort = 1925;
  private final String jarFileName = "surrogate.jar";

  //private InetAddress multicastRequestAddress;
  private final String multicastRequestAddress = "224.0.1.95";
  //private InetAddress myAddress;
  private String myAddress;
  private int unicastResponsePort;

  //private DatagramSocket reqRespSocket;
  private DatagramConnection reqRespSocket;
  //Provo a usare quella sopra sia per mandare che per ricevere
  //private DatagramSocket respSocket;
  //private InetAddress registrationAddress;
  private String registrationAddress;
  private int registrationTCPPort;
  private int registrationUDPPort;

  private boolean accepting = false, accepted = false; //TODO: da sostituire con qualcosa di pi consistente per la sincronizzazione
  private MonitoringAlivementThread monitoringAlivementThread;

  private boolean loadedWrappers = false;
  private boolean searchingPSE = false;
  private boolean foundPSE = false;
  private boolean loggedIn = false;

  Displayable1 displayable1;
  WorkingScreen workingScreen;
  RequestSearchingScreen requestSearchingScreen;
  LoginScreen loginScreen;

  private static final String installedServicesFile = "installed.txt";

  /**
   * Hashtable che mantiene le corrispondenze (Nome del servizio - ServiceDescriptor).
   */
  private Hashtable wrappersHT = new Hashtable();

  public MIDlet mySelf;

  /**Construct the midlet*/
  public MIDlet1() {
    mySelf = this;
  }

  /**Main method*/
  public void startApp() {
    Displayable current = Display.getDisplay(this).getCurrent();
    if (current == null) {
      displayable1 = new Displayable1(this);
      workingScreen = new WorkingScreen(this);
      requestSearchingScreen = new RequestSearchingScreen(this);
      loginScreen = new LoginScreen(this);
      Display.getDisplay(this).setCurrent(displayable1);
    }
    else {
      Display.getDisplay(this).setCurrent(current);
    }
  }

  /**Handle pausing the MIDlet*/
  public void pauseApp() {
  }

  /**Handle destroying the MIDlet*/
  public void destroyApp(boolean unconditional) {
  }

  /**Quit the MIDlet*/
  public void quitApp() {
    System.out.println("quitApp()");
    destroyApp(true);
    notifyDestroyed();
  }

  public void doWrappersLoading() {
    //Mostro un display di attesa
    workingScreen.setMessage("Loading Wrappers...");
    Display.getDisplay(this).setCurrent(workingScreen);
    new Thread() {
      public void run() {
        System.out.println("doWrappersLoading()");
        try {
          findWrappers();
          loadedWrappers = true;
        }
        catch (PSEException ex) {
          System.out.println("Grosso errore nel caricare i wrappers: "+ex.getMessage());
        }
        //mostro il display con i ridultati
        //workingScreen.setMessage("Wrappers loaded!");
        Display.getDisplay(mySelf).setCurrent(requestSearchingScreen);
      }
    }.start();
  }

  private void findWrappers() throws PSEException {
    //Leggi il file installedServicesFile e carica le classi indicate per ottenere informazioni
    InputStream is;
    try {
      is = getClass().getResourceAsStream("/"+installedServicesFile);
    }
    catch (Exception ex) {
      throw new PSEException("Unable to open file "+installedServicesFile+" for reading Wrappers' classes\n"+ex.getMessage());
    }
    String line;
    wrappersHT = new Hashtable();
    Vector wrapperNames = new Vector();
    Hashtable wrapperIndexes = new Hashtable();
    Vector serviceInterfaces = new Vector();
    Hashtable interfacesIndex = new Hashtable();
    int bridgeIndex = 0;
    try {
      line = readLine(is);
    }
    catch (IOException ex) {
      throw new PSEException("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
    }
    while ((line!=null) && (line.length()>0)) {
      System.out.println("Ho letto : "+line);
      Class cl = null;
      ServiceWrapper sw = null;
      try {
        //System.out.println("loading -1");
        //cl = Class.forName("pse.temp.client.Wrapper");
        //System.out.println("Faccio compare = "+line.compareTo("pse.temp.client.Wrapper"));
        cl = Class.forName(line);
        //System.out.println("loading -2");
        sw = (ServiceWrapper)cl.newInstance();
        //System.out.println("loading -3");
      }
      catch (Exception ex) {
        throw new PSEException("Error in loading Wrapper class: "+ex.getMessage());
      }
      System.out.println("Ho trovato : "+sw.getWrapperInfo());
      String proposedName = sw.getWrapperInfo().getServiceName();
      int proposedIndex = 1;
      //Faccio in modo che due servizi non abbiano lo stesso nome
      if (wrapperNames.contains(proposedName)) {
        String key = (String)wrapperNames.elementAt(wrapperNames.indexOf(proposedName));
        Integer oldIndex = (Integer)wrapperIndexes.get(key);
        Integer newIndex = new Integer(oldIndex.intValue()+1);
        wrapperIndexes.remove(key);
        wrapperIndexes.put(key, newIndex);
        proposedName = proposedName+"("+newIndex.intValue()+")";
      }
      else {
        wrapperNames.addElement(proposedName);
        wrapperIndexes.put(proposedName, new Integer(1));
      }
      //Ordino ed assegno un indice unico ai wrapper che fanno riferimento alla stessa interfaccia di servizio
      String servInt = sw.getWrapperInfo().getServiceInterface();
      if (serviceInterfaces.contains(servInt)) {
        String key = (String)serviceInterfaces.elementAt(serviceInterfaces.indexOf(servInt));
        Integer oldIndex = (Integer)interfacesIndex.get(key);
        Integer newIndex = new Integer(oldIndex.intValue()+1);
        interfacesIndex.remove(key);
        interfacesIndex.put(key, newIndex);
        proposedIndex = newIndex.intValue();
      }
      else {
        serviceInterfaces.addElement(servInt);
        interfacesIndex.put(servInt, new Integer(1));
      }
      ServiceDescriptor sd = new ServiceDescriptor(sw);
      sd.setWrapperIndex(proposedIndex);
      sd.setBridgeIndex(bridgeIndex++);
      wrappersHT.put(proposedName, sd);
      try {
        line = readLine(is);
      }
      catch (IOException ex) {
        throw new PSEException("Error reading file "+installedServicesFile+"\n"+ex.getMessage());
      }
    }
  }

  private String readLine(InputStream input) throws IOException {
    boolean eol = false;
    int current;
    StringBuffer buf = new StringBuffer();
    while (!eol) {
      current = input.read();
      //System.out.println("Ho letto "+current);
      if (current == 13) {
        input.read();
        eol = true;
      }
      else if (current == -1) eol = true;
      else {
        buf.append((char)current);
      }
    }
    return new String(buf);
  }

  public void doFrameworkSearch() {
    workingScreen.setMessage("Searching Surrogate host...");
    Display.getDisplay(this).setCurrent(workingScreen);
    searchPSE();
  }

  public void searchPSE() /*throws PSEException*/ {
    //Qua devo avviare tuttalaproceduradel surrogate:
    try {
      findSurrogateHost();
    }
    catch (PSEException ex) {
      workingScreen.appendMessage("Error in finding surrogate host: "+ex.getMessage());
      return;
    }
    workingScreen.appendMessage("Surrogate host found. Registering the surrogate...");
    try {
      registerSurrogate();
    }
    catch (PSEException ex) {
      workingScreen.appendMessage("Error in registering surrogate: "+ex.getMessage());
      return;
    }
    /*if (true) {
      showStatusMessage("Adesso torno subito perch sto testando");
      return;
    }*/
    //findSurrogateHost();
    //registerSurrogate();
    //Aviare ilmessenger
    //invocare messenger.findFramework
    //Creo un DeciceMessenger
    workingScreen.appendMessage("Surrogate registered. Waiting for establishing connection with it...");
    System.out.println("Aspetto che le socket siano state aperte");
    while (!accepted) {
      try {
        Thread.sleep(500);
      }
      catch (Exception ex) {
      }
    }
    System.out.println("Aposto: s1 e s2 sono state aperte");
    messenger = new DeviceMessenger(this, s1, s2);
    //Invoco searchPSE tramite di esso
    //System.out.println("searchPSE - 1");
    workingScreen.appendMessage("Connection established. Searching for PSE Framework in the network");
    try {
      messenger.findFramework();
    }
    catch (PSEException ex) {
      workingScreen.appendMessage("Error in finding Framework");
      return;
    }
    searchingPSE = true;
    workingScreen.appendMessage("PSEFrameWork Found!");
    //System.out.println("searchPSE - 2");
    //DI SEGUITO LA VERSIONE VECCHIA
    //Qui da fare cose tipiche di Jini, ricercando un servizio che implementi PSEInterface
    /*LookupDiscovery discover = null;
    try {
      discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    } catch(Exception e) {
      System.err.println(e.toString());
      //System.exit(1);
      throw new PSEException("Error in finding Lookup Service");
    }
    discover.addDiscoveryListener(this);*/
    Display.getDisplay(this).setCurrent(loginScreen);
  }

  private void findSurrogateHost() throws PSEException {
    try {
      myAddress = getMyAddress();
    } catch (Exception e) {
        System.out.println("Errore nella lettura del mio indirizzo: "+e.getMessage());
      }
    try {
      //reqRespSocket = new DatagramSocket(0);
      //TODO Qua poi invece che 45674 sarebbe meglio fare un ciclo...
      reqRespSocket = (DatagramConnection)InternalConnector.openInternal("datagram://:45674", Connector.READ_WRITE, false);
      //unicastResponsePort =reqRespSocket.getLocalPort();
      unicastResponsePort = 45674;
      //System.out.println("Socket registrata: Indirizzo = "+reqRespSocket.getLocalAddress().getHostAddress());
      //System.out.println("Porta = "+unicastResponsePort);
    } catch (Exception e) {
        System.out.println("Errore nella creazione della socket reqSocket :"+e.getMessage());
        if (reqRespSocket!=null) {
          try {
            reqRespSocket.close();
          }
          catch (Exception ex) {
          }
        }
    }
    try {
      //Mando la richiesta
      System.out.println("Costruisco il datagram packet...");
      //DatagramPacket pkt = marshalOutgoingHostRequest(multicastRequestAddress, multicastRequestPort, unicastResponsePort);
      Datagram pkt = marshalOutgoingHostRequest(multicastRequestAddress, multicastRequestPort, unicastResponsePort);
      System.out.println("ho costruito il datagramma e vale" + pkt.toString());
      System.out.println("Invio il datagramma all'indirizzo di Multicast");
      reqRespSocket.send(pkt);
      System.out.println("Mandata la richiesta. Ora attendo la risposta...");
      byte[] buf = new byte[512];
      //DatagramPacket rpkt = new DatagramPacket(buf, buf.length);
      Datagram rpkt = reqRespSocket.newDatagram(buf, buf.length);
      reqRespSocket.receive(rpkt);
      System.out.println("Ho ricevuto qualcosa. Adesso parso...");
      DataInputStream dis =null;
      try {
        byte[] bytes = rpkt.getData();
        dis = new DataInputStream(new ByteArrayInputStream(bytes));
        IncomingHostResponse resp = new IncomingHostResponse(dis);
        registrationAddress = resp.getHostAddress();
        registrationTCPPort = resp.tcpPort;
        registrationUDPPort=  resp.udpPort;
        System.out.println("Il messaggio che ho ricevuto e' cosi' composto : \nAddress = "+resp.getHostAddress()+"\nTCP port = "+resp.tcpPort+"\nUDP port = "+resp.udpPort);
      } catch (Exception e) {
        System.out.println("Errore in lettura della risposte: "+e.getMessage());
        throw new PSEException("Errore in lettura della risposte: "+e.getMessage());
      }
    } catch (Exception e) {
        System.out.println("Errore in findSurrogateHost() "+e.getMessage());
        throw new PSEException("Errore in findSurrogateHost() "+e.getMessage());
      }
    try {
      reqRespSocket.close();
    }
    catch (Exception ex) {
    }
  }//findSurrogateHost

  private void registerSurrogate() throws PSEException {
    System.out.println("registerSurrogate()");
    //Socket sock = null;
    StreamConnection sock = null;
    //ServerSocket ss = null;
    StreamConnectionNotifier ss = null;
    DataOutputStream dos = null;
    try {
      System.out.println("Apro la socket per registrare il surrogate");
      //sock = new Socket(registrationAddress, registrationTCPPort);
      sock = (StreamConnection)InternalConnector.openInternal("socket://"+registrationAddress+":"+registrationTCPPort, Connector.READ_WRITE, false);
      System.out.println("Ho aperto la connessione di registrazione su : "+"socket://"+registrationAddress+":"+registrationTCPPort);
      //ss = new ServerSocket(0);
      //TODO anche qui come per le datagram-server manca metodo per poi reperire la porta, eventualmente fare ciclo
      ss = (StreamConnectionNotifier)InternalConnector.openInternal("serversocket://:7645", Connector.READ_WRITE, false);
      new ListeningThread(ss).start();
      while (!accepting) {
        try {
          //Pre per mettere alla socket di mettersi in ascolto...
          System.out.println("Dormo un po'");
          Thread.sleep(500);
          System.out.println("Ora mi sveglio");
        }
        catch (Exception ex) {
        }
      }
      // generate initialization data
      System.out.println("Costruisco l'initialization data");
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      dos = new DataOutputStream(bos);
      //dos.writeShort(ss.getLocalPort());
      //NEW Anche se anche in questo caso ci vorrebbe qualcosa di meglio che una porta statica
      dos.writeShort(7645);
      //System.out.println("Ho scritto :"+ss.getLocalPort());
      //NEW
      System.out.println("Ho scritto :"+7645);
      dos.flush();
      byte[] initData = bos.toByteArray();
      System.out.println("Predo il contenuto del file jar");
      //File jarFile = new File(jarFileName);
      //FileInputStream fis = new FileInputStream(jarFile);
      InputStream fis = getClass().getResourceAsStream("/"+jarFileName);
      System.out.println("Ho trovato "+jarFileName+" ? "+fis );
      System.out.println("Scrivo il messaggio sulla socket...");
      //dos = new DataOutputStream(sock.getOutputStream());
      dos = sock.openDataOutputStream();
      dos.writeInt(PROTOCOL_VERSION);
      System.out.println("Scrivo: "+PROTOCOL_VERSION);
      dos.writeShort(0); // set the length of surrogate URL as 0
      System.out.println("Scrivo: 0");
      dos.writeInt(initData.length);
      System.out.println("Scrivo: "+initData.length);
      dos.write(initData, 0, initData.length);
      System.out.println("Scrivo: il contenuto di initData");
      // write the jar out
      //dos.writeInt((int)jarFile.length());
      //System.out.println("Scrivo: "+(int)jarFile.length());
      //NEW Bisogna che prima scandisca tutto il jar:
      System.out.println("Conto da quanti bytes  costituito il jar...");
      byte[] buff = new byte[1024];
      int count = 0;
      int len = fis.read(buff);
      while (len > 0) {
          //dos.write(buff, 0, len);
          count+=len;
          len = fis.read(buff);
      }
      dos.writeInt(count);
      System.out.println("Scrivo: "+count);
      //NEW Apro e richiudo. Ho finito di contare: questa volta scrivo proprio sulla socket
      fis.close();
      fis = getClass().getResourceAsStream("/"+jarFileName);
      len = fis.read(buff);
      while (len > 0) {
          dos.write(buff, 0, len);
          System.out.println("Ne scrivo : "+len);
          len = fis.read(buff);
      }
      dos.flush();
      System.out.println("Scrivo: il contenuto del jar file");
    }
    catch (Exception ex) {
      System.out.println("Errore nella fase ri registrazione del surrogate: "+ex.getMessage());
      try {
          if (dos != null)
            dos.close();
          if (sock != null)
              sock.close();
          /*if (ss != null)
            ss.close();*/
      } catch(IOException e) {}
      throw new PSEException("Errore nella fase ri registrazione del surrogate: "+ex.getMessage());
    }
    //ATT.NE! Questo  fondamentale! Altrimenti madison lancia socketException (ma perch poi ?!?!?)
    try {
        if (dos != null)
          dos.close();
        if (sock != null)
            sock.close();
        /*if (ss != null)
          ss.close();*/
    } catch(IOException e) {}
  }//registerSurrogate


  private String getMyAddress() {
    //Versione temporanea. Poi bisogner prevedere in altro modo, per. es. tramite il server MidpIpAddress
    return "me";
  }

  /**
   * Costruisce un Datagram di Multicast Host Request
   * @param hostAddres indirizzo multicast a cui mandare la richiesta
   * @param porta a cui mandare la richiesta
   * @param porta su cui si attendono risposte
   */
  //private DatagramPacket marshalOutgoingHostRequest(InetAddress hostAddr, int reqPort, int respPort) throws IOException {
  //NEW
  private Datagram marshalOutgoingHostRequest(String hostAddr, int reqPort, int respPort) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    DataOutputStream dos = new DataOutputStream(bos);
    dos.writeInt(PROTOCOL_VERSION);
    //String addr = deviceAddress.getHostAddress();
    String addr = myAddress;
    dos.writeShort(addr.length());
    //dos.writeBytes(addr);
    //NEW
    dos.write(addr.getBytes());
    dos.writeShort(respPort);
    dos.flush();
    byte[] message = bos.toByteArray();
    /*if (message.length > Constants.MAX_MULTICAST_PKT_SIZE)
        throw new IllegalArgumentException("device address marshals " +
                                           "too large");*/
    //return new Datagram(message, message.length, hostAddr, reqPort);
    //NEW
    return reqRespSocket.newDatagram(message, message.length, "datagram://"+hostAddr+":"+reqPort);
  }

  private class IncomingHostResponse {
    //private InetAddress hostAddress;
    //NEW
    private String hostAddress;
    private int tcpPort;
    private int udpPort;

    /** Parse a response message from a host. */
    IncomingHostResponse(DataInputStream dis) throws IOException {
      int version = dis.readInt();
      if (version != PROTOCOL_VERSION)
        throw new IOException("unsupported protocol version: " + version);
      // host address
      int addrLen = dis.readUnsignedShort();
      if (addrLen > 0) {
        byte[] addrData = new byte[addrLen];
        dis.readFully(addrData);
        //hostAddress = InetAddress.getByName(new String(addrData));
        //NEW
        hostAddress = new String(addrData);
      } else {
          /*if (System.getProperty("com.sun.jini.madison.debug") != null)
              System.err.println("Host response with no host address!");*/
          throw new IOException("Host response with no host address!");
      }
      tcpPort = dis.readUnsignedShort();
      udpPort = dis.readUnsignedShort();
      dis.close();
    }

    /*public InetAddress getHostAddress() {
        return hostAddress;
    }*/
    //NEW
    public String getHostAddress() {
      return hostAddress;
    }

    public int getTCPPort() {
      return tcpPort;
    }

    public int getUDPPort() {
      return udpPort;
    }

    /*public boolean equals(Object o) {
        if (!(o instanceof IncomingHostResponse))
            return false;
        IncomingHostResponse resp = (IncomingHostResponse)o;
        return hostAddress.equals(resp.hostAddress) &&
            tcpPort == resp.tcpPort && udpPort == resp.udpPort;
    }*/
  }//IncomingHostResponse

  private class ListeningThread extends Thread {
    //ServerSocket servSock;
    StreamConnectionNotifier servSock;

    ListeningThread(StreamConnectionNotifier servSock) {
      this.servSock = servSock;
    }

    public void run() {
      try {
        accepting =true;
        System.out.println("Sono ListeningSocket. Sto per fare accept()");
        //s1 = servSock.accept();
        s1 = servSock.acceptAndOpen();
        System.out.println("Sono ListeningSocket. Ho fatto la prima accept()");
        //s2 = servSock.accept();
        s2 = servSock.acceptAndOpen();
        System.out.println("Sono ListeningSocket. Ho fttto la seconda accept()");
        accepted= true;
        servSock.close();
      } catch (Exception e) {
          System.out.println("Errore nella accept: "+e.getMessage());
      } finally {
          try {
            servSock.close();
          }
          catch (Exception ex) {
            System.out.println("Errore nella chiusura di ss");
          }
      }
    }
  }//ListeningThread

   /**
   * Metodo per il keepAlive. Deve rispondere con il messaggio di KeepAliveSuccess ed
   * eventualmente prevedere un thread per controlare che entro il long arrivi un'altra richiesta di
   * keep alive (per ora questo non viene fatto)
   */
  public void keepAlive(long period) {
    System.out.println("Invocato Client01.keepAlive("+period+")");
    try {
      messenger.deviceAlive();
    }
    catch (Exception ex) {
      System.out.println("Error in signalling alivement to surrogate");
    }
    if (monitoringAlivementThread == null) {
      monitoringAlivementThread = new MonitoringAlivementThread(System.currentTimeMillis()+period+10000);
      monitoringAlivementThread.start();
    }
    else {
      monitoringAlivementThread.setNextCheck(System.currentTimeMillis()+period+10000); /*10 secondi di margine*/
    }
  }

  class MonitoringAlivementThread extends Thread {
    Object sem = new Object();
    long nextCheck;
    MonitoringAlivementThread(long nextCheckMillis) {
      synchronized (sem) {
        nextCheck = nextCheckMillis;
      }
    }
    public void setNextCheck(long nextCheckMillis) {
      synchronized (sem) {
        nextCheck = nextCheckMillis;
      }
    }
    public void run () {
      long currentTime = System.currentTimeMillis();
      Thread currentThread = Thread.currentThread();
      while (currentThread == monitoringAlivementThread) {
        currentTime = System.currentTimeMillis();
        long tempNextCheck = nextCheck;
        System.out.println("Itero: currentTime = "+currentTime+" tempNextCheck = "+tempNextCheck);
        if (tempNextCheck>currentTime) {
          try {
            Thread.sleep(tempNextCheck-currentTime);
          }
          catch (InterruptedException ex) {
          }
        }
        else {
          System.out.println("Non rilevato in tempo messaggio di alivement");
          //TODO fare qlcs di pi?
          monitoringAlivementThread = null;
        }
      }
    }
  }//MonitoringAlivementThread

  public void foundFramework() {
    foundPSE = true;
    searchingPSE = false;
    System.out.println("Invocato foundFramework");
  }

  public void doLogin(String un, String pwd) {
    System.out.println("Richiesto Login con "+un+"-"+pwd);
    if (!loadedWrappers) { //In realt questa verifica  inutile
      System.out.println("Wrappers not loaded yet");
      return;
    }
    if (!foundPSE) {
      System.out.println("Framework not found yet");
      return;
    }
    if (loggedIn) {
      System.out.println("Session already active. Please logout before login again");
      return;
    }
    try {
      messenger.login(un, pwd, new DeviceProfileJ2ME());
    }
    catch (PSEException ex) {
      System.out.println("Error in login: "+ex.getMessage());
      return;
    }
    loggedIn = true;
    //loggedIn();
  }


  public void doExitAtPrompt() {}
  public void doStart() {}
  public void doBackFromSteps() {}
  public void doExitAtInitial() {}
  public void doViewWrapperInfo(String wrapperName) {}
  public void doSearchFramework() {}
  public void doBackFromOfflineWrapperInfo() {}
  public void doChoseFramework() {} //Poi qua bisogner passare indicativo
  public void doBackFromChoosingFramework() {}
  public void doViewFrameworkInfo() {}
  public void doBackFromLogin() {}
  public void doLogout() {}
  public void doHandleService(String wrapperName) {}
  public void doChangeProfile(String wrapperName) {}
  public void doBackFromWrapperInfo() {}
  public void doUseService(String wrapperName) {}
  public void doViewActiveWrapperInfo(String wrapperName) {}
  /**Metodi comuni a tutti i WrapperMenu*/
  public void doBackFromWrapperMenu() {}
  /**Metodi da AloneWrapperMenuScreen*/
  public void doFindProxies(String m) {}
  public void doStartService(String WrapperName, int proxyIndex) {}
  /**Metodi da RunningWrapperMenuScreen*/
  public void doPauseService(String wrapperName) {}
  public void doStopService(String wrapperName) {}
  public void doResumeService(String wrapperName) {}
  public boolean showScreen(Displayable d, String wn) {
  return false;
  }
}