package pse.client.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;
import pse.shared.framework.*;
import pse.client.*;

import java.net.*;
import java.io.*;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.*;

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;

public class SurrogateTemp implements SurrogateMessengerListener, DiscoveryListener, BridgeContext {

  Socket s1, s2;
  SurrogateMessenger messenger;
  private PSEInterface pseInterface = null;
  //TODO: decidere: Questi mantenerlo solo qui, anche qui o qui no?
  private SessionID sessionID = null;
  String userName = null, password = null;
  private Map bridges = Collections.synchronizedMap(new TreeMap()); //bridgeIndex (Integer) - BridgeDescriptor
  private boolean deviceAlive;

  public SurrogateTemp() {
    deviceAlive = true;
    Runtime.getRuntime().addShutdownHook(
      new Thread () {
        public void run() {
          System.out.println("Sono HOOK!");
          deactivate();
        }
      }
    );
  }

  public void activate() {//Giusto per bellezza
    //Apre la serverSocket ...
    try {
      ServerSocket ss = new ServerSocket(12345);
      s1 = ss.accept();
      s2 = ss.accept();
      ss.close();
    }
    catch (IOException ex) {
      System.out.println("Errore nella creazione delle socket");
    }
    //Imposta il SurrogateMessenger
    messenger = new SurrogateMessenger(this, s2, s1);
  }
  public static void main(String[] args) {
    System.setSecurityManager(new RMISecurityManager());
    SurrogateTemp surrogateTemp1 = new SurrogateTemp();
    surrogateTemp1.activate();
  }

  public void deactivate() {
    if (messenger != null) messenger.destroy();
    if (s1!=null) {
      try {
        s1.close();
      }
      catch (IOException ex) {
      }
    }
    if (s2!=null) {
      try {
        s2.close();
      }
      catch (IOException ex) {
      }
    }
  }

  public void findFramework() throws PSEException {
    System.out.println("Sono SurrogateTemp.findServices()");
    //Qui da fare cose tipiche di Jini, ricercando un servizio che implementi PSEInterface
    LookupDiscovery discover = null;
    try {
      discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    } catch(Exception e) {
      System.err.println(e.toString());
      //System.exit(1);
      throw new PSEException("Error in finding Lookup Service");
    }
    discover.addDiscoveryListener(this);
  }

  public void discovered(DiscoveryEvent evt) {

    ServiceRegistrar[] registrars = evt.getRegistrars();
    Class [] classes = new Class[] {PSEInterface.class};
    ServiceTemplate template = new ServiceTemplate(null, classes,
                                                   null);
    boolean found = false;

    for (int n = 0; (n < registrars.length) && (!found); n++) {
      System.out.println("Service found");
      ServiceRegistrar registrar = registrars[n];
      try {
        pseInterface = (PSEInterface) registrar.lookup(template);
      } catch(java.rmi.RemoteException e) {
        //e.printStackTrace();
        System.out.println("Errore 1: "+e.getMessage());
        continue;
      }
      if (pseInterface == null) {
        System.out.println("Classifier null");
        continue;
      }
      else {
        found = true;
        try {
          messenger.foundFramework();
        }
        catch (PSEException ex) {
          System.out.println("Errore1 "+ex.getMessage());
        }
      }
    }
  }

  public void discarded(DiscoveryEvent evt) {
    // empty
  }

  public void login(String userName, String password, DeviceProfile deviceProfile) throws PSEException {
    if (pseInterface == null) {
      throw new PSEException("Surrogate hasn't found framewok yet");
    }
    if (sessionID != null) {
      throw new PSEException("Surrogate has an active session");
    }
    try {
      sessionID = pseInterface.login(userName, password, deviceProfile);
      this.userName = userName;
      this.password = password;
    }
    catch (RemoteException ex) {
      throw new PSEException(ex.getMessage());
    }
    //return sessionID;
  }

  public void logout() throws PSEException {
    if (pseInterface == null) {
      throw new PSEException("Surrogate hasn't found framewok yet");
    }
    if (sessionID == null) {
      throw new PSEException("Surrogate has no active session");
    }
    try {
      pseInterface.logout(userName, password, sessionID);
    }
    catch (RemoteException ex) {
      throw new PSEException(ex.getMessage());
    }
    //TODO: stoppare tutti i serviiz attivi o non permettere il logout se ci sono servizi attivi
    sessionID = null; userName = null; password = null;
  }

  public FoundServicesProfiles findServices(int bridgeIndex, String serviceInterface, int wrapperIndex, WrapperProfile wrapperProfile, String bridgeName) throws PSEException {
    System.out.println("SurrogateTemp.findServices()v1 con "+bridgeIndex+serviceInterface+wrapperIndex+wrapperProfile);
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se no lo creo nuovo. Altrimenti inserisco tali valori anche se probabilmente dovrei segnalare errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (bridges.containsKey(brIndex)) {
      //Aggiorno
      bd = (BridgeDescriptor)bridges.get(brIndex);
    }
    else {
      //Faccio nuovo
      bd = new BridgeDescriptor();
      bridges.put(brIndex, bd);
    }
    bd.setServiceInterface(serviceInterface);
    bd.setWrapperIndex(wrapperIndex);
    bd.setWrapperProfile(wrapperProfile);
    bd.setBridgeName(bridgeName);
    //invoco il metodo su pseInterface
    FoundServices foundServices = null;
    try {
      foundServices = pseInterface.findService(sessionID, serviceInterface, wrapperIndex, wrapperProfile);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in choosing Proxy : "+ex.getMessage());
    }
    if (foundServices == null) {
      throw new PSEException("Error in choosing Proxy : Empty");
    }
    if (foundServices.getNumberOfServices() == 0) {
      throw new PSEException("No matching proxy found");
    }
    bd.setFoundServices(foundServices);
    FoundServicesProfiles res = new FoundServicesProfiles(foundServices);
    return res;
  }

  public FoundServicesProfiles findServices(int bridgeIndex) throws PSEException {
    System.out.println("SurrogateTemp.findServices()v2 con "+bridgeIndex);
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    String serviceInterface = bd.getServiceInterface();
    int wrapperIndex = bd.getWrapperIndex();
    WrapperProfile wrapperProfile = bd.getWrapperProfile();
    //invoco il metodo su pseInterface
    FoundServices foundServices = null;
    try {
      System.out.println("sessionID = "+sessionID+" serviceInterface = "+serviceInterface+" wrapperIndex = "+wrapperIndex+" wrapperProfile = "+wrapperProfile);
      foundServices = pseInterface.findService(sessionID, serviceInterface, wrapperIndex, wrapperProfile);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in choosing Proxy : "+ex.getMessage());
    }
    if (foundServices == null) {
      throw new PSEException("Error in choosing Proxy : Empty");
    }
    if (foundServices.getNumberOfServices() == 0) {
      throw new PSEException("No matching proxy found");
    }
    bd.setFoundServices(foundServices);
    FoundServicesProfiles res = new FoundServicesProfiles(foundServices);
    return res;
  }

  public int startBridge(int bridgeIndex, int proxyIndex) throws PSEException {
    System.out.println("Sono startBridge di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    FoundServices fs = bd.getFoundServices();
    if (fs == null) {
      throw new PSEException("No proxies found for this service");
    }
    ServiceProxy sp = fs.getService(proxyIndex);
    if (sp == null) {
      throw new PSEException("No proxies present at index "+proxyIndex);
    }
    String brName = bd.getBridgeName();
    if (brName == null) {
      throw new PSEException("No bridge name for this bridge index");
    }
    ServiceBridge sb = null;
    try {
      System.out.println("Cerco di caricare la classe "+brName);
      Class sbClass = Class.forName(brName);
      System.out.println("Ora creo una nuova istanza");
      sb = (ServiceBridge)sbClass.newInstance();
    }
    catch (Exception ex) {
      throw new PSEException("Error in loading bridge class: "+ex);
    }
    if (sb == null) {
      throw new PSEException("No ServiceBridge loaded for this bridge index");
    }
    bd.setServiceBridge(sb);
    int brPort;
    try {
      brPort = sb.start(sp, this);
    }
    catch (PSEException ex) {
      sb.stop();
      throw new PSEException("Error in starting bridge: "+ex.getMessage());
    }
    return brPort;
  }

  public void stopBridge(int bridgeIndex) throws PSEException {
    System.out.println("Sono stopBridge di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    ServiceBridge sb = null;
    sb = bd.getServiceBridge();
    if (sb == null) {
      throw new PSEException("No ServiceBridge loaded for this bridge index");
    }
    sb.stop();
  }

  public void startedService(int bridgeIndex) throws PSEException {
    System.out.println("Sono startedService di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    String si = bd.getServiceInterface();
    if (bd == null) {
      throw new PSEException("No interface specified for service");
    }
    int wi = bd.getWrapperIndex();
    ServiceSessionID sSID;
    /*
    ServiceSessionID sSID = bd.getServiceSessionID();
    if (sSID == null) {
      throw new PSEException("No service session active");
    }
    */
    try {
      sSID = pseInterface.startedService(sessionID, si, wi);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    bd.setServiceSessionID(sSID);
  }

  public void stoppedService(int bridgeIndex) throws PSEException {
    System.out.println("Sono startedService di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    ServiceSessionID sSID = bd.getServiceSessionID();
    if (sSID == null) {
      throw new PSEException("No service session active");
    }
    try {
      pseInterface.stoppedService(sSID);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  public UserProfile getUserProfile() throws PSEException {
    System.out.println("Sono getUserProfile() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    UserProfile up = null;
    try {
      up = pseInterface.getUserProfile(sessionID);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    return up;
  }

  public void setUserProfile(UserProfile userProfile) throws PSEException {
    System.out.println("Sono setUserProfile() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    try {
      pseInterface.setUserProfile(sessionID, userProfile);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  public ServicePreferences getServicePreferences(String serviceInterface) throws PSEException {
    System.out.println("Sono getServicePreferences() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    ServicePreferences sp = null;
    try {
      sp = pseInterface.getServicePreferences(sessionID, serviceInterface);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    return sp;
  }

  public void setServicePreferences(ServicePreferences servicePreferences) throws PSEException {
    System.out.println("Sono setServicePreferences() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    try {
      pseInterface.setServicePreferences(sessionID, servicePreferences);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  /*public void setBridges(int [] bridgeIndexes, String [] serviceInterfaces, int [] wrapperIndexes) throws PSEException {
    if (!((bridgeIndexes.length==serviceInterfaces.length)&&(bridgeIndexes.length==wrapperIndexes.length)))
      throw new PSEException("Dimensions of arrays don't match");
    bridges.clear();
    for (int i=0; i<bridgeIndexes.length; i++) {
      int bridgeIndex = bridgeIndexes[i];
      String serviceInterface = serviceInterfaces[i];
      int wrapperIndex = wrapperIndexes[i];
      //TODO...
    }
  }*/

  public ProfileHandler getProfileHandler() throws PSEException {
    try {
      return pseInterface.getProfileHandler();
    } catch (RemoteException e) {
      throw new PSEException("ProfileHandler not found : "+e.getMessage());
    }
  }

  public void deviceAlive() {
    //TODO Qua settare la veriabile a true
    deviceAlive = true;
  }

}