package pse.client.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.framework.*;
import pse.shared.*;
import pse.client.*;

import java.io.*;
import java.util.*;

import java.rmi.*;
import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;

 /*
 I compiti che deve realizzare il client sono:
 -gestione dell'interfaccia grafica
 -caricamento e gestione dei wrapper installati sul dispositivo
 -Importante! Per client j2me trattazione della questione surrogate
 -Ritrovamento come semplice client dell'interfaccia pseInterface, attraverso la quale avverr tutta l'interazione con il framework

 Interfaccia Grafica:
  All'utente deve essere consentito in successione:
  -di ricercare il framework di adattamento nella rete corrente
  -se trovato di fare login
  -di selezionare un servizio alla volta da eseguire (M.D. E' possibile avere pi servizi attivi allo stesso momento? su j2me?)
  -di fare una nuova ricerca per un servizio (mod. trigger; in seguito faremo anche possibile notifica di eventi)
  -di terminare un servizio attivo
  -di effettuare il logout

  Problemi:
  -cosa fare se ci sono pi reti, pi Lookup Services, pi Framework accessibili?
  -Trattare bene e linearmente il rapporto con l'interfaccia grafica (ciostrutturare bene i metodi nelle due interfaccie
    e la loro semantica
  -Bisogna considerare tutte le cose/errori che possono succedere in qualsiasi punto dell'elaborazione
    e trattarli opportunamente

 */

public class Client implements IClientUIListener, DiscoveryListener, ProfileHandlerFinder {

  private IClientUI ui = null;
  //private static String installedServicesFile = "installed.txt";
  private static String installedServicesFile = "c:\\temp\\installed.txt";

  /**
   * Hashtable che mantiene le corrispondenze (Nome del servizio - ServiceWrapper).
   * Per ora  consentito per ogni nome dell'interfaccia avere un sono ServiceWrapper
   * TODO: Permettere che per un servizio sul cliente siano installati pi wrappers
   */
  private Hashtable wrappersHT = null;

  private PSEInterface pseInterface = null;
  private SessionID sessionID = null;
  private String userName = null, password = null;

  public Client() {
  }

  public void setUI(IClientUI ui) {
    this.ui = ui;
  }

  public static void main(String[] args) {
    System.setSecurityManager(new RMISecurityManager());
    Client client = new Client();
    ClientUI ui = new ClientUI(client);
    client.setUI(ui);
    ui.setSize(600, 400);
    ui.setTitle("Client PSE");
    //ui.pack();
    ui.setVisible(true);
  }

  public void searchFramework() {
    try {
      searchPSE();
    }
    catch (PSEException ex) {
      ui.showStatusMessage(ex.getMessage());
    }
  }

  public void searchPSE() throws PSEException {
    //Qui da fare cose tipiche di Jini, ricercando un servizio che implementi PSEInterface
    LookupDiscovery discover = null;
    try {
      discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    } catch(Exception e) {
      System.err.println(e.toString());
      //System.exit(1);
      throw new PSEException("Error in finding Lookup Service");
    }
    discover.addDiscoveryListener(this);
  }

  public void discovered(DiscoveryEvent evt) {

    ServiceRegistrar[] registrars = evt.getRegistrars();
    Class [] classes = new Class[] {PSEInterface.class};
    ServiceTemplate template = new ServiceTemplate(null, classes,
                                                   null);
    boolean found = false;

    for (int n = 0; (n < registrars.length) && (!found); n++) {
      System.out.println("Service found");
      ServiceRegistrar registrar = registrars[n];
      try {
        pseInterface = (PSEInterface) registrar.lookup(template);
      } catch(java.rmi.RemoteException e) {
        //e.printStackTrace();
        System.out.println("Errore 1: "+e.getMessage());
        continue;
      }
      if (pseInterface == null) {
        System.out.println("Classifier null");
        continue;
      }
      else {
        found = true;
        ui.foundFramework();
      }
    }
  }

  public void discarded(DiscoveryEvent evt) {
    // empty
  }

  public void login(String userName, String pwd) {
    if (pseInterface == null) {
      ui.showStatusMessage("Framework not found yet");
      return;
    }
    if (sessionID != null) {
      ui.showStatusMessage("Session already active. Please logout before login again");
      return;
    }
    try {
      sessionID = pseInterface.login(userName, pwd, new DeviceProfile());
      this.userName = userName;
      this.password = pwd;
    }
    catch (RemoteException ex) {
      ui.showStatusMessage("Error in login: "+ex.getMessage());
      return;
    }
    ui.loggedIn();
  }

  public void logout() {
    if (pseInterface == null) {
      ui.showStatusMessage("Framework not found yet");
      return;
    }
    if (sessionID == null) {
      ui.showStatusMessage("None Session active");
      return;
    }
    try {
      pseInterface.logout(userName, password, sessionID);
    }
    catch (RemoteException ex) {
      ui.showStatusMessage("Error in logout: "+ex.getMessage());
      return;
    }
    sessionID = null; userName = null; password = null;
    ui.loggedOut();
  }

  public void findWrappers() { //throws Exception
    //Leggi il file installedServicesFile e carica le classi indicate per ottenere informazioni
    try {
      FileInputStream fin = new FileInputStream(installedServicesFile);
      BufferedReader in = new BufferedReader(new InputStreamReader(fin));
      String line;
      wrappersHT = new Hashtable();
      Vector wrapperNames = new Vector();
      while ((line=in.readLine())!=null) {
        //System.out.println("Ho letto : "+line);
        Class cl = Class.forName(line);
        ServiceWrapper sw = (ServiceWrapper)cl.newInstance();
        System.out.println("Ho trovato : "+sw.getWrapperInfo());
        wrappersHT.put(sw.getWrapperInfo().getServiceName(), sw);
        wrapperNames.add(sw.getWrapperInfo().getServiceName());
      }
      ui.setWrapperList(wrapperNames);

    }
    catch (Exception ex) {

    }
  }

  public void startService(String ServiceName) {
    ServiceWrapper sw = (ServiceWrapper)wrappersHT.get(ServiceName);
    if (sw==null) {
      ui.showStatusMessage("Wrapper not found!");
      return;
    }
    //TODO:Ci vorrebbe anche controllo su sessionID, ecc..
    FoundServices foundServices = null;
    try {
      foundServices = pseInterface.findService(sessionID, sw.getWrapperInfo().getServiceInterface(), 0, sw.getWrapperProfile());
    }
    catch (RemoteException ex) {
      ui.showStatusMessage("Error in choosing Proxy : "+ex.getMessage());
      System.out.println("Error in choosing Proxy : "+ex.getMessage());
      return;
    }
    if (foundServices == null) {
      ui.showStatusMessage("Error in choosing Proxy : Empty");
      return;
    }
    System.out.println("Sono startService(). Sono stati scaricati i proxies. Ora se ce n'e' piu' d'uno dovrei sceglierlo e poi iniziare l'elaborazione");
    System.out.println("La descrizione dei servizi trovati e':\n"+foundServices.toString()+"Lancio il primo...");
    if (foundServices.getNumberOfServices() >0) {
      ServiceSessionID sSID = null;
      try {
        sSID = pseInterface.startedService(sessionID, sw.getWrapperInfo().getServiceInterface(), 0);
      }
      catch (RemoteException ex) {
        ui.showStatusMessage("Error in notifying service starting : "+ex.getMessage());
        System.out.println("Error in notifying service starting : "+ex.getMessage());
        return;
      }
      sw.start((ServiceProxy)foundServices.getService(0), this, sSID);
      try {
        Thread.sleep(1000);
      }
      catch (InterruptedException ex) {
      }


      //TODO STOP
    }
    //sw.start(proxy);
  }

  public void getServiceInfo(String ServiceName) {
  }

  //METODI DI ProfileHandlerFinder

  public ProfileHandler getProfileHandler() throws PSEException {
    //TODO: richiederlo attraverso PSEInterface
    try {
      return pseInterface.getProfileHandler();
    } catch (RemoteException e) {
      throw new PSEException("ProfileHandler not found : "+e.getMessage());
    }
  }

  public void notifyFinished(ServiceSessionID sSID) {
  }
}