package pse.server.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.framework.*;
import pse.shared.*;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

public class PSEInterfaceImpl extends UnicastRemoteObject implements RemotePSEInterface {

  private LoginModule loginModule;
  private CacheModule cacheModule;
  private SessionModule sessionModule;
  private ProfileHandler profileHandler;

  public PSEInterfaceImpl(LoginModule loginModule, CacheModule cacheModule, SessionModule sessionModule) throws RemoteException{
    if ((loginModule==null)||(cacheModule==null)||(sessionModule==null)) {
      throw new RemoteException("Error in parameters");
    }
    this.loginModule = loginModule;
    this.cacheModule = cacheModule;
    this.sessionModule = sessionModule;
    profileHandler = new ProfileHandlerImpl(sessionModule);
  }

  public PSEServerInfo getInfo() throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.getInfo()");
    return new PSEServerInfo();
  }

  public SessionID login(String userName, String password, DeviceProfile deviceProf) throws RemoteException{
    System.out.println("Sono PSEInterfaceImpl.login(). Richiesto login con : "+userName+" - "+password);
    ClientProfile personalProf = null;
    SessionID sID;
    try {
      personalProf = loginModule.login(userName, password);
    }
    catch (PSEException ex) {
      throw new RemoteException("Error in login : "+ex.getMessage());
    }
    try {
      sID = sessionModule.createNewSession(userName, personalProf, deviceProf);
    }
    catch (PSEException ex) {
      throw new RemoteException("Error in creating session : "+ex.getMessage());
    }
    loginModule.addSession(userName, sID);
    return sID;
  }

  public void logout(String userName, String password, SessionID sID) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.logout(). Richiesto logout dalla sessione "+sID);
    try {
      sessionModule.removeSession(sID);
    }
    catch (PSEException ex) {
      throw new RemoteException("Error in removing session : "+ex.getMessage());
    }
    loginModule.removeSession(sID);
  }

  //
  //Metodi per la gestione dei servizi
  //

  public FoundServices findService(SessionID sID, String serviceInterface, int index, WrapperProfile prof) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.findService()v.1 . Sessione = "+sID+" Interfaccia del servizio = "+serviceInterface+" Profilo = "+prof);
    //TODO : REgistrazione di prof presso il SessionModule
    //e quindi richiesta sempre a SessionModule del profilo che mi interessa
    //Qualcosa tipo:
    //sessionModule.updateProfile(SessionID, prof)
    //sessionModule.getProfile(SessionID, Servizio che mi serve)
    prof.setWrapperIndex(index);
    try {
      sessionModule.updateProfile(sID, prof);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
    SpecificClientProfile profForSearch;
    try {
      profForSearch = sessionModule.getProfile(sID, serviceInterface, index);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
    FoundServices found = null;
    try {
      found = cacheModule.findServices(profForSearch, serviceInterface);
    } catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
    //if (found==null) -> Eccezione?
    return found;
  }

  public FoundServices findService(SessionID sID, String serviceInterface, int index) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.findService()v.2. Sessione = "+sID+" Interfaccia del servizio = "+serviceInterface);
    //TODO: In questo caso, rispetto al caso precedente non c' la fase di update del profilo presso il SessionProfile
    SpecificClientProfile profForSearch;
    try {
      profForSearch = sessionModule.getProfile(sID, serviceInterface, index);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
    FoundServices found = null;
    try {
      found = cacheModule.findServices(profForSearch, serviceInterface);
    } catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
    //if (found==null) -> Eccezione?
    return found;
  }

  //A dire la verit questo  inutile, se si trasferiscono tutti i wrappers con findServices
  public Object chooseService(SessionID sID, String serviceInterface, int ident) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.chooseService(). Sessione = "+sID+" Interfaccia del servizio = "+serviceInterface+" Identificatore proxy = "+ident);
    return null;
  }

  public ServiceSessionID startedService(SessionID sID, String serviceInterface, int ident) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.startedService(). Sessione = "+sID+" Interfaccia del servizio = "+serviceInterface+" Identificatore proxy = "+ident);
    ServiceSessionID sSID = null;
    try {
      sSID = sessionModule.startedService(sID,serviceInterface, ident);
    }
    catch (PSEException ex) {
      throw new RemoteException("Error in notifying service start : "+ex.getMessage());
    }
    return sSID;
  }

  public void stoppedService(ServiceSessionID sSID) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.stoppedService(). sSID = "+sSID);
    try {
      sessionModule.stoppedService(sSID);
    }
    catch (PSEException ex) {
      throw new RemoteException("Error in notifying service stop : "+ex.getMessage());
    }
  }

  public ProfileHandler getProfileHandler() throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.getProfileHandler()");
    return profileHandler;
  }

  public UserProfile getUserProfile(SessionID sID) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.getUserProfile(). sID = "+sID);
    try {
      //return sessionModule.getUserProfile(sID);
      return loginModule.getUserProfile(sID);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
  }

  public void setUserProfile(SessionID sID, UserProfile userProfile) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.setUserProfile(). sID = "+sID+" userProfile = "+userProfile);
    try {
      //sessionModule.setUserProfile(sID, userProfile);
      loginModule.setUserProfile(sID, userProfile);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
  }

  public ServicePreferences getServicePreferences(SessionID sID, String serviceInterface) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.getServicePreferences(). sID = "+sID+" serviceInterface = "+serviceInterface);
    try {
      return loginModule.getServicePreferences(sID, serviceInterface);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
  }

  public void setServicePreferences(SessionID sID, ServicePreferences servicePreferences) throws RemoteException {
    System.out.println("Sono PSEInterfaceImpl.setServicePreferences(). sID = "+sID+" servicePreferences = "+servicePreferences);
    try {
      loginModule.setServicePreferences(sID, servicePreferences);
    }
    catch (PSEException ex) {
      throw new RemoteException(ex.getMessage());
    }
  }

}
