package pse.server.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;

import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.core.lease.Lease;
// import com.sun.jini.lease.LeaseRenewalManager;
// import com.sun.jini.lease.LeaseListener;
// import com.sun.jini.lease.LeaseRenewalEvent;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;
import java.rmi.RMISecurityManager;

import java.util.Vector;
import java.rmi.RemoteException;


public class Server implements DiscoveryListener, LeaseListener {

  protected PSEInterfaceImpl impl;
  protected LeaseRenewalManager leaseManager = new LeaseRenewalManager();

  private LoginModule loginModule;
  private CacheModule cacheModule;
  private SessionModule sessionModule;

  protected Vector leases = new Vector();

  private ServerUI ui;

  public static void main(String argv[]) {
      new Server();

      // no need to keep server alive, RMI will do that
  }

  public Server() {

    // install suitable security manager
    System.setSecurityManager(new RMISecurityManager());

    LookupDiscovery discover = null;
    try {
        discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    } catch(Exception e) {
        System.err.println(e.toString());
        System.exit(1);
    }
    sessionModule = new SessionModule();
    loginModule = new LoginModule(sessionModule);
    try {
      cacheModule = new CacheModule(/*discover*/);
    }
    catch (PSEException ex) {
      System.out.println("Error in creating CacheModule: "+ex.getMessage());
      System.exit(1);
    }



    try {
        impl = new PSEInterfaceImpl(loginModule, cacheModule, sessionModule);
    } catch(Exception e) {
        System.err.println("New impl: " + e.toString());
        System.exit(1);
    }

    discover.addDiscoveryListener(this);

    Runtime.getRuntime().addShutdownHook(
      new Thread () {
        public void run() {
          System.out.println("Sono HOOK!");
          for (int i = 0; i<leases.size(); i++) {
            try {
              ((Lease)leases.elementAt(i)).cancel();
            }
            catch (Exception ex) {
              continue;
            }
          }
        }
      }
    );

    LoginModuleUI lmui = new LoginModuleUI(loginModule);
    SessionModuleUI smui = new SessionModuleUI(sessionModule);
    CacheModuleUI cmui = new CacheModuleUI(cacheModule);
    try {
      cacheModule.startSearch(discover);
    } catch (PSEException e) {
      System.out.println("Error in starting CacheModule: "+e.getMessage());
      System.exit(1);
    }
    ui = new ServerUI(lmui, smui, cmui);
    ui.setSize(640, 480);
    ui.setVisible(true);
  }

  public void discovered(DiscoveryEvent evt) {

      ServiceRegistrar[] registrars = evt.getRegistrars();
      //RemotePSEInterface service;

      for (int n = 0; n < registrars.length; n++) {
          ServiceRegistrar registrar = registrars[n];

          // export the proxy service
          ServiceItem item = new ServiceItem(null,
                                             impl,
                                             null);
          ServiceRegistration reg = null;
          try {
              reg = registrar.register(item, Lease.FOREVER);
          } catch(java.rmi.RemoteException e) {
              System.err.print("Register exception: "+e.getMessage());
              //e.printStackTrace();
              // System.exit(2);
              continue;
          }
          try {
              System.out.println("service registered at " +
                                 registrar.getLocator().getHost());
          } catch(Exception e) {
          }
          leases.add(reg.getLease());
          leaseManager.renewUntil(reg.getLease(), Lease.FOREVER, this);
      }
  }

  public void discarded(DiscoveryEvent evt) {

  }

  public void notify(LeaseRenewalEvent evt) {
      System.out.println("Lease expired " + evt.toString());
  }
}

