package pse.shared;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

//TODO : Poi oltre a set e get fare gli update, i cancel, etc
/*
Note:
-Si garantisce solo che esista la struttura per mantenere le informazioni sui servizi, mentre tutti i componenti-profili possono essere anche null
*/

import java.util.Vector;
import java.io.Serializable;

public class ClientProfile implements Serializable {

  private String name;
  private DeviceProfile deviceProfile;
  private UserProfile userProfile;
  private ServiceSettingList serviceSettingList;

  public ClientProfile(String name, DeviceProfile deviceProfile, UserProfile userProfile,
                        ServicePreferences servicePreferences[], WrapperProfile wrapperProfile[]) {
    this.name = name;
    this.deviceProfile = deviceProfile;
    this.userProfile = userProfile;
    serviceSettingList = new ServiceSettingList();
    if (servicePreferences!=null) {
      for (int i=0; i<servicePreferences.length; i++) {
        serviceSettingList.addServicePreferences(servicePreferences[i]);
      }
    }
    if (wrapperProfile != null) {
      for (int i=0; i<wrapperProfile.length; i++) {
        serviceSettingList.addWrapperProfile(wrapperProfile[i]);
      }
    }
  }

  /**
   * Essendo ClientProfile un contenitore di moduli che invece all'interno del framework
   * non possono essere modificati ma solo aggiunti, risulta necessario fare un costruttore
   * di questo tipo che copia tutti tali componenti non condivicendo per la struttura della classe
   */
  public ClientProfile(ClientProfile copy) {
    /*this.name = copy.name;
    this.deviceProfile = copy.deviceProfile;
    this.userProfile = copy.userProfile;
    serviceSettingList = new ServiceSettingList();*/
    this(copy.name, copy.deviceProfile, copy.userProfile, copy.getAllServicePreferences(), copy.getAllWrapperProfile());
  }

  public ClientProfile(String name) {
    this(name, null, null, new ServicePreferences [0], new WrapperProfile [0]);
  }

  public void setName(String n) {
    name = n;
  }

  public String getName() {
    return name;
  }

  public void setDeviceProfile(DeviceProfile deviceProfile) {
    this.deviceProfile = deviceProfile;
  }

  public DeviceProfile getDeviceProfile() {
    return deviceProfile;
  }

  public void setUserProfile(UserProfile userProfile) {
    this.userProfile = userProfile;
  }

  public UserProfile getUserProfile() {
    return userProfile;
  }

  public void setServicePreferences(ServicePreferences sp) {
    serviceSettingList.addServicePreferences(sp);
  }

  public void setServicePreferences(ServicePreferences [] sp) {
    if (sp!=null) {
      for (int i=0; i<sp.length; i++) {
        serviceSettingList.addServicePreferences(sp[i]);
      }
    }
  }

  public void addWrapperProfile(WrapperProfile wp) {
    serviceSettingList.addWrapperProfile(wp);
  }

  public void addWrapperProfile(WrapperProfile [] wp) {
    if (wp != null) {
      for (int i=0; i<wp.length; i++) {
        serviceSettingList.addWrapperProfile(wp[i]);
      }
    }
  }

  public String [] getServicesWithProfile() {
    return serviceSettingList.getServiceNames();
  }

  public ServicePreferences getServicePreferences(String serviceInterface) {
    return serviceSettingList.getServicePreferences(serviceInterface);
  }

  public WrapperProfile [] getWrapperProfile(String serviceInterface) {
    return serviceSettingList.getWrapperProfile(serviceInterface);
  }

  public WrapperProfile getWrapperProfile(String serviceInterface, int index) {
    return serviceSettingList.getWrapperProfile(serviceInterface, index);
  }

  //Da usare per la ui. Poi fare anche metodi pi selettivi, per es, per un solo servizio, ecc
  /*public ServiceSettingList getServiceSettingList() {
    return serviceSettingList;
  }*/

  public String toString() {
    return "ClientProfile:toString() non implementato";
  }

  private ServicePreferences [] getAllServicePreferences() {
    return serviceSettingList.getAllServicePreferences();
  }

  private WrapperProfile [] getAllWrapperProfile() {
    return serviceSettingList.getAllWrapperProfile();
  }

}

/**
 * Rappresenta una lista dei profili dei wrappers. Garantisce che pi wrappers non abbiano lo stesso indice.
 * Non controlla per che appartengano alla stessa interfaccia. Tale controllo deve essere effettuato a livello
 * superiore, ovvero dall'utilizzatore di tale classe
 */
class WrapperProfileList implements Serializable {
  //TODO ora non  implementato che l'indice del vettore corrisponde all'index del wrapper
  private Vector wrappers;
  WrapperProfileList() {
    wrappers = new Vector();
  }
  /**
   * Aggiunge un wrapper. Se l'indice  gi presente lo sostituisce
   * @param wp il wrapper da aggiungere
   */
  public void addWrapperProfile(WrapperProfile wp) {
    //int index =  wp.getWrapperIndex();
    //wrappers.add(index, wp);
    //Controlla prima che tale indice non sia presente e se c' lo cancella
    if (wp!=null) {
      WrapperProfile swp = getWrapperProfile(wp.getWrapperIndex());
      if (swp!=null) wrappers.removeElement(swp);
      wrappers.add(wp);
    }
  }
  //TODO: Per ora ritorna null se non c'. Pensare se  il caso di lanciare un'eccezione
  /**
   * Restituisce il wrapper con l'indice specificato
   * @param index l'indice del wrapper da ricercare
   * @return il wrapper con indice index
   */
  public WrapperProfile getWrapperProfile(int index) {
    //return (WrapperProfile)wrappers.get(index);
    for (int i=0; i<wrappers.size(); i++) {
      WrapperProfile wp = (WrapperProfile)wrappers.elementAt(i);
      if (wp.getWrapperIndex()==index) return wp;
    }
    return null;
  }
  /**
   * Restituisce tutti i wrappers
   * @return la lista di tutti i wrappers
   */
  public WrapperProfile [] getAllWrappers() {
    int wpn = wrappers.size();
    WrapperProfile [] wp = new WrapperProfile[wpn];
    for (int i=0; i<wrappers.size(); i++) {
      wp[i]=(WrapperProfile)wrappers.elementAt(i);
    }
    return wp;
  }
}//WrapperProfileList

/**
 * Racchiude tutte le informazioni relative ad una interfaccia di servizio
 */
class ServiceSettings implements Serializable {
  /** l'interfaccia del servizio di riferimento */
  private String serviceInterface;
  /** la lista dei wrappers */
  private WrapperProfileList wrapperList;
  /** le preferenzedell'utente riguardo al servizio*/
  private ServicePreferences preferences;
  /**
   * Costruttore.
   * @param serviceInterface l'interfaccia del servizio di riferimento
   */
  ServiceSettings(String serviceInterface) {
    this.serviceInterface = serviceInterface;
    wrapperList = new WrapperProfileList();
  }
  public String getServiceInterface() {
    return serviceInterface;
  }
  public void setWrapperProfileList(WrapperProfileList wpl) {
    if (wpl!=null) wrapperList = wpl;
    else wrapperList = new WrapperProfileList();
  }
  public WrapperProfileList getWrapperProfileList() {
    return wrapperList;
  }
  public void setServicePreferences(ServicePreferences sp) {
    preferences = sp;
  }
  public ServicePreferences getServicePreferences() {
    return preferences;
  }
}//ServiceSettings

/**
 * Rappresenta una lista di ServiceSettings
 * Questo controlla che per ogni interfaccia esista una sola entry nella lista
 */
class ServiceSettingList implements Serializable {
  private Vector list;
  ServiceSettingList () {
    list = new Vector();
  }
  /**
   * Aggiunge un ServiceSettings. Se l'interfacciadel servizio  gi presente sostuisce quella
   */
  public void addServiceSettings(ServiceSettings settings) {
    if (settings!=null) {
      ServiceSettings ss = getServiceSettings(settings.getServiceInterface());
      if (ss!=null) {
        list.remove(ss);
      }
      list.add(settings);
    }
  }
  /**
   * Aggiunge delle preferenze utente relative ad un servizio
   */
  public void addServicePreferences(ServicePreferences sp) {
    if (sp!=null) {
      String spInterface = sp.getServiceInterface();
      ServiceSettings servSettings = getServiceSettings(spInterface);
      if (servSettings == null) {//bisogna crearne uno nuovo
        servSettings = new ServiceSettings(spInterface);
        servSettings.setServicePreferences(sp);
        addServiceSettings(servSettings);
      }
      else {//E' gi presente, bisogna aggiornarlo
        servSettings.setServicePreferences(sp);
      }
    }
  }
  /**
   * Aggiunge un WrapperProfile. Evita i doppioni
   */
  public void addWrapperProfile(WrapperProfile wp) {
    if (wp!=null) {
      String spInterface = wp.getServiceInterface();
      ServiceSettings servSettings = getServiceSettings(spInterface);
      if (servSettings == null) {//bisogna crearne uno nuovo
        //System.out.println("Nuova interfaccia");
        servSettings = new ServiceSettings(spInterface);
        servSettings.getWrapperProfileList().addWrapperProfile(wp);
        addServiceSettings(servSettings);
      }
      else {//E' gi presente, bisogna aggiornarlo
        //System.out.println("Vecchia interfaccia");
        servSettings.getWrapperProfileList().addWrapperProfile(wp);
      }
    }
  }
  public ServiceSettings getServiceSettings(String serviceInterface) {
    //System.out.println("ServiceSettings.getServiceSettings("+serviceInterface+")");
    for (int i=0; i<list.size(); i++) {
      ServiceSettings css = (ServiceSettings)list.elementAt(i);
      if (css!=null) {
        String ccsInterface = css.getServiceInterface();
        if (serviceInterface.equals(ccsInterface)) return css;
      }
    }
    return null;
  }
  public ServicePreferences getServicePreferences(String serviceInterface) {
    for (int i=0; i<list.size(); i++) {
      ServiceSettings css = (ServiceSettings)list.elementAt(i);
      if (css!=null) {
        String ccsInterface = css.getServiceInterface();
        if (serviceInterface.equals(ccsInterface)) return css.getServicePreferences();
      }
    }
    return null;
  }
  public WrapperProfile [] getWrapperProfile(String serviceInterface) {
    for (int i=0; i<list.size(); i++) {
      ServiceSettings css = (ServiceSettings)list.elementAt(i);
      if (css!=null) {
        String ccsInterface = css.getServiceInterface();
        if (serviceInterface.equals(ccsInterface)) return css.getWrapperProfileList().getAllWrappers();
      }
    }
    return null;
  }
  public WrapperProfile getWrapperProfile(String serviceInterface, int index) {
    for (int i=0; i<list.size(); i++) {
      ServiceSettings css = (ServiceSettings)list.elementAt(i);
      if (css!=null) {
        String ccsInterface = css.getServiceInterface();
        if (serviceInterface.equals(ccsInterface)) return css.getWrapperProfileList().getWrapperProfile(index);
      }
    }
    return null;
  }
  public String [] getServiceNames() {
    int nserv = list.size();
    String temp [] = new String[nserv];
    for (int i = 0; i<nserv; i++) {
      temp[i]=((ServiceSettings)list.elementAt(i)).getServiceInterface();
    }
    return temp;
  }
  public Vector getAllServiceSettings() {
    return list;
  }
  ServicePreferences [] getAllServicePreferences() {
    Vector spVector = new Vector();
    for (int i=0; i<list.size(); i++) {
      ServiceSettings ss = (ServiceSettings)list.elementAt(i);
      ServicePreferences sp = ss.getServicePreferences();
      spVector.add(sp);
    }
    ServicePreferences [] ret = new ServicePreferences[spVector.size()];
    for (int i=0; i<spVector.size(); i++) {
      ret[i]=(ServicePreferences)spVector.elementAt(i);
    }
    return ret;
  }
  WrapperProfile [] getAllWrapperProfile() {
    Vector wpVector = new Vector();
    for (int i=0; i<list.size(); i++) {
      ServiceSettings ss = (ServiceSettings)list.elementAt(i);
      WrapperProfile wp [] = ss.getWrapperProfileList().getAllWrappers();
      for (int j=0; j<wp.length; j++) {
        wpVector.add(wp[i]);
      }
    }
    WrapperProfile [] ret = new WrapperProfile[wpVector.size()];
    for (int i=0; i<wpVector.size(); i++) {
      ret[i]=(WrapperProfile)wpVector.elementAt(i);
    }
    return ret;
  }
}//ServiceSettingList
