package pse.shared.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;

import java.rmi.RemoteException;

 /**
  * E' l'interfaccia principale usata dal client per accedere al framework
  */
public interface PSEInterface {

  ///////////////////////////////////////////////
  //Metodi per ricavare informazioni sul server//
  ///////////////////////////////////////////////

  /**
   * Per ottenere informazioni riguardo al server PSE
   * @return informazioni riguardo al server PSE
   */
  public PSEServerInfo getInfo() throws RemoteException;


  ///////////////////////////////////////
  //Metodi per login/logout dell'utente//
  ///////////////////////////////////////

  /**
   * Effettua il login nel sistema. Se l'utente  autorizzato ne vengono trovati
   * i profili presenti in rete e gli viene associato un identificativo unico di sessione
   *
   * @param userName lo user name dell'utente
   * @param password la password dell'utente
   * @param prof  il componente di profilo che descrive HW, SO, ecc...
   * @return l'identificativo unico di sessione associato
   */
  public SessionID login(String userName, String password, DeviceProfile prof) throws RemoteException;

  /**
   * Termina la sessione corrente. Come parametro sarebbe sufficiente l'identificativo di sessione,
   * ma per motivi di sicurezza si passano anche userName e password per asssicurare
   * che colui che termina la sessione  effettivamente colui che l'ha instaurata
   *
   * @param userName lo user name dell'utente
   * @param password la password dell'utente
   * @param sID l'identificativo unico di sessione associato
   */
  public void logout(String userName, String password, SessionID sID) throws RemoteException;

  //////////////////////////////////////
  //Metodi per la gestione dei servizi//
  //////////////////////////////////////

  /**
   * Questo metodo effettua la prima scrematura, scegliendo i proxy adatti al profilo
   * Se va a buon fine (ovvero se non viene lanciata nessuna eccezione), sul server
   * vinene associato alla sessione l'insieme di proxy trovati
   * @param sID l'identificativo unico della sessione corrente
   * @param serviceName il nome della interfaccia del servizio
   * @param indice unico del wrapper per quell'interfaccia sul client
   * @param prof il componente del profilo che descrive il wrapper per il servizio
   * @return
   */
  FoundServices findService(SessionID sID, String serviceInterface, int index, WrapperProfile prof) throws RemoteException;

  /**
   * Altra versione del metodo precedente, in cui per il profilo del wrapper  gi stato comunicato
   * in precedenza
   */
  FoundServices findService(SessionID sID, String serviceInterface, int index) throws RemoteException;

  //A dire la verit questo  inutile, se si trasferiscono tutti i wrappers con findServices
  //Object chooseService(SessionID sID, String serviceInterface, int ident) throws RemoteException;

  //////////////////////////////////////
  //Metodi per la gestione del profilo//
  //////////////////////////////////////

  //Qualcosa tipo:
  //public void changeProfile(Profile prof) throws RemoteException;
  //o remove, update, ecc per rendere pi modulare la questione, magari agendo direttamente su Sections e Attributes

  /**
   * Ottiene lo UserProfile dal server
   * @param sID la sessione corrente
   * @return lo UserProfile dell'utente
   */
  public UserProfile getUserProfile(SessionID sID) throws RemoteException;

  /**
   * Setta lo UserProfile dell'utente
   * @param sID la sessione corrente
   * @param userProfile lo UserProfile da settare
   */
  public void setUserProfile(SessionID sID, UserProfile userProfile) throws RemoteException;

  /**
   * Ottiene le ServicePreferences relative ad un servizio
   * @param sID l'identificativo della sessione
   * @param serviceInterface l'interfaccia del servizio
   * @return le preferenze dell'utente relativamente a tale servizio
   */
  public ServicePreferences getServicePreferences(SessionID sID, String serviceInterface) throws RemoteException;

  /**
   * Imposta le preferenze dell'utente relativamente ad un servizio
   * @param sID l'identificativo della sessione
   * servicePreferences le preferenze dell'utente relativamente ad un servizio. L'indicazione dell'interfaccia  mantenuta all'interno di questo oggetto
   */
  public void setServicePreferences(SessionID sID, ServicePreferences servicePreferences) throws RemoteException;

  //Inoltre metodo per poter reperire il profilo durante lo svolgimento del servizio
  ProfileHandler getProfileHandler() throws RemoteException;

  /////////////////////////////////////////////
  //Metodi per la gestione dei servizi attivi//
  /////////////////////////////////////////////

  /**
   * Notifica al server l'inizio dell'esecuzione di un servizio, ottenendo un identificativo unico di tale esecuzione
   * @param sID l'identificativo unico di sessione
   * @param serviceInterface l'interfaccia del servizio
   * @param ident l'indice del wrapper
   * @return identificativo unico dell'esecuzione
   */
  ServiceSessionID startedService(SessionID sID, String serviceInterface, int ident) throws RemoteException;

  /**
   * Notifica al server la fine dell'esecuzione di un servizio
   * @param sSID identificativo unico dell'esecuzione del servizio
   */
  void stoppedService(ServiceSessionID sSID) throws RemoteException;

}