package pse.client;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import java.util.Vector;
import java.io.*;

/**
 * Rappresenta una collezione di attributi (ProfileAttributeJ2ME) e di sottosezioni (ProfileSectionJ2ME)
 */
public class ProfileSectionJ2ME {

  /**
   * Il nome della sezione
   */
  private String name;
  /**
   * La lista delle sottosezioni
   */
  private Vector profileSectionList;
  /**
   * La lista degli attributi
   */
  private Vector profileAttributeList;

  /**
   * Costruttore vuoto. Pone il nome della sezione alla string vuota.
   */
  public ProfileSectionJ2ME() {
    name = "";
    profileSectionList = new Vector();
    profileAttributeList = new Vector();
  }

  /**
   * Costruttore con nome di sezione
   * @param n nome della sezione
   */
  public ProfileSectionJ2ME(String n) {
    name =n;
    profileSectionList = new Vector();
    profileAttributeList = new Vector();
  }

  /**
   * Costruttore con nome, lista di attributi e sotto-sezioni
   * @param n nome della sezioni
   * @param psl lista delle sottosezioni
   * @param pal lista degli attributi
   */
  public ProfileSectionJ2ME(String n, ProfileSectionJ2ME[] psl, ProfileAttributeJ2ME[] pal) {
    name = n;
    profileSectionList = new Vector();
    profileAttributeList = new Vector();
    int i;
    if (psl != null) {
      for (i=0; i<psl.length; i++) {
        if (psl[i]!=null)
          profileSectionList.add(psl[i]);
      }
    }
    if (pal != null) {
      for (i=0; i<pal.length; i++) {
        if (pal[i]!=null)
          profileAttributeList.add(pal[i]);
      }
    }
  }

  /**
   * Test di sezione vuota
   * @return true se la sezione non contiene n attributi n sottosezioni
   */
  public boolean isNull() {
    return (profileSectionList.isEmpty() && profileAttributeList.isEmpty());
  }

  /**
   * Setta il nome di sezione
   * @param str il nome della sezione
   */
  public void setName(String str) {
    name = str;
  }

  /**
   * Restituisce il nome di sezione
   * @return il nome della sezione
   */
  public String getName() {
    return name;
  }

  /**
   * Restituisce il numero di sottosezioni
   * @return il numero di sottosezioni
   */
  public int getNumberOfSection() {
    return profileSectionList.size();
  }

  /**
   * Restituisce una determinata sottosezione
   * @param index indice della sottosezione nella lista
   * @return la sottosezione selezionata
   */
  public ProfileSectionJ2ME getSection(int index) {
    return (ProfileSectionJ2ME)profileSectionList.get(index);
  }

  /**
   * Cancella una sottosezione
   * @param index indice della sottosezione nella lista
   */
  public void delSection(int index) {
    profileSectionList.remove(index);
  }

  /**
   * Cancella una sottosezione
   * @param ps sottosezione da cancellare
   */
  public void delSection(ProfileSectionJ2ME ps) {
    if (ps!=null) {
      for (int i=0; i<profileSectionList.size(); i++) {
        if (ps == profileSectionList.elementAt(i))
          profileSectionList.remove(i);
      }
    }
  }

  /**
   * Aggiunge una sottosezione
   * @param ps sottosezione da aggiungere. L'aggiunta viene fatta in coda alla lista
   */
  public void addSection(ProfileSectionJ2ME ps) {
    if (ps!=null) profileSectionList.add(ps);
  }

  /**
   * Aggiunge una lista di sottosezioni
   * @param psl lista delle sottosezioni da aggiungere. Vengono aggiunti solo le sottosezioni non null
   */
  public void addSectionList(ProfileSectionJ2ME[] psl) {
    if (psl != null) {
      for (int i=0; i<psl.length; i++) {
        if (psl[i]!=null)
          profileSectionList.add(psl[i]);
      }
    }
  }

  /**
   * Restituisce la lista di sotto sezioni
   * @return la lista delle sottosezioni
   */
  public ProfileSectionJ2ME[] getSectionList() {
    int psn = getNumberOfSection();
    ProfileSectionJ2ME [] psl = new ProfileSectionJ2ME[psn];
    for (int i=0; i<psn; i++) {
      psl[i] = (ProfileSectionJ2ME)profileSectionList.get(i);
    }
    return psl;
  }

  /**
   * Seleziona una sezione per nome
   * @param na il nome della sottosezione da scegliere. Il confronto viene effettuato
   * utilizzando il metodi equals() di String. Viene restituito il primo match corretto
   * e non viene effettuata la ricerca ricorsiva (cio si valutano solo le sottosezioni
   * di primo livello)
   * @return la sottosezione con il nome specificato o null se non  stato trovato
   */
  public ProfileSectionJ2ME getSectionByName(String na) {
    //TODO: devo fare ricerca ricorsiva anche nelle sottosezioni?
    //TODO: Perch per le sezioni solo uno e per gli attributi una lista?
    int psn = getNumberOfSection();
    for (int i=0; i<psn; i++) {
      ProfileSectionJ2ME ps = (ProfileSectionJ2ME)profileSectionList.get(i);
      String psName = ps.getName();
      if (psName.equals(na)) return ps;
    }
    return null;
  }

  /*public ProfileSection[] getSectionByName(String na) {
    //TODO
    return null;
  }*/

  /**
   * Restituisce il numero di attributi
   * @return il numeor di attributi
   */
  public int getNumberOfAttribute() {
    return profileAttributeList.size();
  }

  /**
   * Restituisce una determinato attributo
   * @param index indice dell'attributo nella lista
   * @return l'attributo sekezionato
   */
  public ProfileAttributeJ2ME getAttribute(int index) {
    return (ProfileAttributeJ2ME)profileAttributeList.get(index);
  }

  /**
   * Cancella un attributo
   * @param index indice dell'attributo nella lista
   */
  public void delAttribute(int index) {
    profileAttributeList.remove(index);
  }

  /**
   * Cancella un attributo
   * @param pa attributo da cancellare
   */
  public void delAttribute(ProfileAttributeJ2ME pa) {
    if (pa!=null) {
      for (int i=0; i<profileAttributeList.size(); i++) {
        if (pa == profileAttributeList.elementAt(i))
          profileAttributeList.remove(i);
      }
    }
  }

  /**
   * Aggiunge un attributo
   * @param pa attributo da aggiungere. L'aggiunta viene fatta in coda alla lista e solo se l'attributo non  null
   */
  public void addAttribute(ProfileAttributeJ2ME pa) {
    if (pa!=null) profileAttributeList.add(pa);
  }

  /**
   * Aggiunge una lista di attributi
   * @param pal lista gegli attributi da aggiungere. Vengono aggiunti solo gli attributi non null
   */
  public void addAttributeList(ProfileAttributeJ2ME[] pal) {
    if (pal != null) {
      for (int i=0; i<pal.length; i++) {
        if (pal[i]!=null)
          profileAttributeList.add(pal[i]);
      }
    }
  }

  /**
   * Restituisce la lista degli attributi
   * @return la lista degli attributi
   */
  public ProfileAttributeJ2ME[] getAttributeList() {
    int pan = getNumberOfAttribute();
    ProfileAttributeJ2ME [] pal = new ProfileAttributeJ2ME[pan];
    for (int i=0; i<pan; i++) {
      pal[i] = (ProfileAttributeJ2ME)profileAttributeList.get(i);
    }
    return pal;
  }

  /**
   * Seleziona un attributo per nome
   * @param attrName il nome dell'attributo da scegliere. Il confronto viene effettuato
   * utilizzando il metodi equals() di String. Vengono restituiti tutti i match corretti
   * @param recursiveSearch specifica se la ricerca deve essere fatta ricorsivamente anche nelle sottosezioni
   * @return l'array contenente tutti gli attributi selezionati (eventualmente un array vuoto)
   */
  public ProfileAttributeJ2ME[] getAttributeByName(String attrName, boolean recursiveSearch) {
    if (recursiveSearch) return getAttributeByName(attrName);
    else return getAttributeByNameNoRec(attrName);
  }

  /**
   * Restituisce gli attributi con quel nome. Effettua una ricerca ricorsiva anche nelle sottosezioni
   * @param na nome dell'attributo
   * @return gli attributi con quel nome. Effettua una ricerca ricorsiva anche nelle sottosezioni
   */
  private ProfileAttributeJ2ME[] getAttributeByName(String na) {
    Vector pal = new Vector();
    getAttributeByNameRec(na, pal);
    int pan = pal.size();
    ProfileAttributeJ2ME [] palVector = new ProfileAttributeJ2ME[pan];
    for (int i=0; i<pan; i++) {
      palVector[i] = (ProfileAttributeJ2ME)profileAttributeList.get(i);
    }
    return palVector;
  }

  /**
   * Trova gli attributi con quel nome. Effettua una ricerca ricorsiva anche nelle sottosezioni
   * @param attrName nome dell'attributo
   * @param res vettore in cui inserire i match trovati,
   */
  private void getAttributeByNameRec(String attrName, Vector res) {
    int pan = getNumberOfAttribute();
    for (int i=0; i<pan; i++) {
      ProfileAttributeJ2ME pa = (ProfileAttributeJ2ME)profileAttributeList.get(i);
      String paName = pa.getName();
      if (paName.equals(attrName)) res.add(pa);
    }
    int psn = getNumberOfSection();
    for (int i=0; i<psn; i++) {
      ProfileSectionJ2ME ps = (ProfileSectionJ2ME)profileSectionList.get(i);
      ps.getAttributeByNameRec(attrName, res);
    }
  }

  /**
   * Restituisce gli attributi con quel nome. Fa la ricerca solo nella sezione corrente non considerando le sottosezioni
   * @param na nome dell'attributo
   * @return gli attributi con quel nome. Fa la ricerca solo nella sezione corrente non considerando le sottosezioni
   */
  private ProfileAttributeJ2ME[] getAttributeByNameNoRec(String na) {
    Vector res = new Vector();
    int pan = getNumberOfAttribute();
    for (int i=0; i<pan; i++) {
      ProfileAttributeJ2ME pa = (ProfileAttributeJ2ME)profileAttributeList.get(i);
      String paName = pa.getName();
      if (paName.equals(na)) res.add(pa);
    }
    pan = res.size();
    ProfileAttributeJ2ME [] palVector = new ProfileAttributeJ2ME[pan];
    for (int i=0; i<pan; i++) {
      palVector[i] = (ProfileAttributeJ2ME)profileAttributeList.get(i);
    }
    return palVector;
  }

  //appiattimento della struttura del profilo
  /*public FlatedAttributeList flatStructure() {
  }
  */

  public String toString() {
    StringBuffer stemp = new StringBuffer();
    for (int i=0; i<profileSectionList.size(); i++) {
      stemp.append(profileSectionList.elementAt(i).toString());
    }
    for (int i=0; i<profileAttributeList.size(); i++) {
      stemp.append(profileAttributeList.elementAt(i).toString());
    }
    return "["+name+stemp+"]";
  }

  public static void writeToStream(ProfileSectionJ2ME sec, DataOutputStream stream) throws IOException {
    //Scrivo il nome della sezione
    stream.writeUTF(sec.getName());
    //Scrivo un intero che indica il numero di attributi
    int na = sec.getNumberOfAttribute();
    stream.writeInt(na);
    //Quindi scrivo tutti gli attributi
    for (int i=0; i<na; i++) {
      ProfileAttributeJ2ME.writeToStream(sec.getAttribute(i), stream);
    }
    //Scrivo un intero che indica il numero di sezioni
    int ns = sec.getNumberOfSection();
    stream.writeInt(ns);
    //Quinid scrivo tutte le sezioni
    for (int i=0; i<ns; i++) {
      ProfileSectionJ2ME.writeToStream(sec.getSection(i), stream);
    }
  }

  public static ProfileSectionJ2ME readFromStream(DataInputStream stream) throws IOException {
    //Leggo il nome della sezione
    String n = stream.readUTF();
    //Leggo da quanti attributi  composto il profilo
    int na = stream.readInt();
    //Ora leggo tutti gli attributi mettendoli nell'array
    ProfileAttributeJ2ME [] pal = new ProfileAttributeJ2ME[na];
    for (int i=0; i<na; i++) {
      ProfileAttributeJ2ME pa = ProfileAttributeJ2ME.readFromStream(stream);
      pal[i] = pa;
    }
    //Leggo da quante sezioni  composto il profilo
    int ns = stream.readInt();
    //Ora leggo tutte le sezioni mettendole nell'array
    ProfileSectionJ2ME [] psl = new ProfileSectionJ2ME[ns];
    for (int i=0; i<ns; i++) {
      ProfileSectionJ2ME ps = ProfileSectionJ2ME.readFromStream(stream);
      psl[i] = ps;
    }
    //Ora creo la sezione da restituire
    return new ProfileSectionJ2ME(n, psl, pal);
  }

}