package pse.client.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;
import pse.shared.framework.*;
import pse.client.*;

import java.net.*;
import java.io.*;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.*;

import net.jini.surrogate.*;
import net.jini.surrogate.ip.*;

//import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryManagement;
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;

public class PSESurrogate implements SurrogateMessengerListener, DiscoveryListener, BridgeContext, Surrogate {

  Socket s1, s2; //s1 = dev to surrogate; s2 = surrogate to dev
  SurrogateMessenger messenger;
  private PSEInterface pseInterface = null;
  //TODO: decidere: Questi mantenerlo solo qui, anche qui o qui no?
  private SessionID sessionID = null;
  String userName = null, password = null;
  private Map bridges = Collections.synchronizedMap(new TreeMap()); //bridgeIndex (Integer) - BridgeDescriptor
  private boolean deviceAlive;
  private boolean deviceConnected;

  private HostContext hostContext = null;
  private IPInterconnectContext ipInterconnectContext =  null;
  private ConnectingThread connectingThread =  null;


  public PSESurrogate() {
    deviceAlive = true;
    deviceConnected = false;
  }

  public void activate(HostContext hostContext, Object context) throws Exception {
    //Controllo  i parametri
    if (hostContext == null) throw new Exception();
    this.hostContext = hostContext;
    if (context == null) throw new Exception();
    ipInterconnectContext = (IPInterconnectContext)context;
    ipInterconnectContext.setKeepAliveHandler(new KeepAliveHandlerImpl());
    connectingThread = new ConnectingThread(this);
    connectingThread.start();
  }

  public void deactivate() {
    System.out.println("Sono deactivate");
    if (messenger != null) messenger.destroy();
    if (s1!=null) {
      try {
        s1.close();
      }
      catch (IOException ex) {
      }
    }
    if (s2!=null) {
      try {
        s2.close();
      }
      catch (IOException ex) {
      }
    }
    //Libero tutte  le risorse mantenute dai bridges
    Iterator iter = bridges.keySet().iterator();
    while (iter.hasNext()) {
      BridgeDescriptor bd = (BridgeDescriptor)bridges.get(iter.next());
      if (bd != null) {
        ServiceBridge sb = bd.getServiceBridge();
        if (sb != null) sb.stop();
      }
    }
    //TODO Comunico anche l'uscita al PSEServer???? Da decidere in un ambito pi ampio
  }

  public void findFramework() throws PSEException {
    System.out.println("Sono SurrogateTemp.findServices()");
    //Qui da fare cose tipiche di Jini, ricercando un servizio che implementi PSEInterface
    /*
    LookupDiscovery discover = null;
    try {
      discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    } catch(Exception e) {
      System.err.println(e.toString());
      //System.exit(1);
      throw new PSEException("Error in finding Lookup Service");
    }
    */
    DiscoveryManagement discover = hostContext.getDiscoveryManager();
    discover.addDiscoveryListener(this);
  }

  public void discovered(DiscoveryEvent evt) {

    ServiceRegistrar[] registrars = evt.getRegistrars();
    Class [] classes = new Class[] {PSEInterface.class};
    ServiceTemplate template = new ServiceTemplate(null, classes,
                                                   null);
    boolean found = false;

    for (int n = 0; (n < registrars.length) && (!found); n++) {
      System.out.println("Service found");
      ServiceRegistrar registrar = registrars[n];
      try {
        pseInterface = (PSEInterface) registrar.lookup(template);
      } catch(java.rmi.RemoteException e) {
        //e.printStackTrace();
        System.out.println("Errore 1: "+e.getMessage());
        continue;
      }
      if (pseInterface == null) {
        System.out.println("Classifier null");
        continue;
      }
      else {
        found = true;
        try {
          messenger.foundFramework();
        }
        catch (PSEException ex) {
          System.out.println("Errore1 "+ex.getMessage());
        }
      }
    }
  }

  public void discarded(DiscoveryEvent evt) {
    // empty
  }

  public void login(String userName, String password, DeviceProfileJ2ME deviceProfile) throws PSEException {
    if (pseInterface == null) {
      throw new PSEException("Surrogate hasn't found framewok yet");
    }
    if (sessionID != null) {
      throw new PSEException("Surrogate has an active session");
    }
    try {
      sessionID = pseInterface.login(userName, password, ProfileConverter.raise(deviceProfile));
      this.userName = userName;
      this.password = password;
    }
    catch (RemoteException ex) {
      throw new PSEException(ex.getMessage());
    }
    //return sessionID;
  }

  public void logout() throws PSEException {
    if (pseInterface == null) {
      throw new PSEException("Surrogate hasn't found framewok yet");
    }
    if (sessionID == null) {
      throw new PSEException("Surrogate has no active session");
    }
    try {
      pseInterface.logout(userName, password, sessionID);
    }
    catch (RemoteException ex) {
      throw new PSEException(ex.getMessage());
    }
    //TODO: stoppare tutti i serviiz attivi o non permettere il logout se ci sono servizi attivi
    sessionID = null; userName = null; password = null;
  }

  public FoundServicesProfiles findServices(int bridgeIndex, String serviceInterface,
            int wrapperIndex, WrapperProfileJ2ME wrapperProfileJ2ME, String bridgeName) throws PSEException {
    System.out.println("SurrogateTemp.findServices()v1 con "+bridgeIndex+serviceInterface+wrapperIndex+wrapperProfileJ2ME);
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se no lo creo nuovo. Altrimenti inserisco tali valori anche se probabilmente dovrei segnalare errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (bridges.containsKey(brIndex)) {
      //Aggiorno
      bd = (BridgeDescriptor)bridges.get(brIndex);
    }
    else {
      //Faccio nuovo
      bd = new BridgeDescriptor();
      bridges.put(brIndex, bd);
    }
    bd.setServiceInterface(serviceInterface);
    bd.setWrapperIndex(wrapperIndex);
    WrapperProfile wp = ProfileConverter.raise(wrapperProfileJ2ME);
    bd.setWrapperProfile(wp);
    bd.setBridgeName(bridgeName);
    //invoco il metodo su pseInterface
    FoundServices foundServices = null;
    try {
      foundServices = pseInterface.findService(sessionID, serviceInterface, wrapperIndex, wp);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in choosing Proxy : "+ex.getMessage());
    }
    if (foundServices == null) {
      throw new PSEException("Error in choosing Proxy : Empty");
    }
    if (foundServices.getNumberOfServices() == 0) {
      throw new PSEException("No matching proxy found");
    }
    bd.setFoundServices(foundServices);
    //FoundServicesProfiles res = new FoundServicesProfiles(foundServices);
    FoundServicesProfiles res = FoundServiceConverter.lower(foundServices);
    return res;
  }

  public FoundServicesProfiles findServices(int bridgeIndex) throws PSEException {
    System.out.println("SurrogateTemp.findServices()v2 con "+bridgeIndex);
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    String serviceInterface = bd.getServiceInterface();
    int wrapperIndex = bd.getWrapperIndex();
    WrapperProfile wrapperProfile = bd.getWrapperProfile();
    //invoco il metodo su pseInterface
    FoundServices foundServices = null;
    try {
      System.out.println("sessionID = "+sessionID+" serviceInterface = "+serviceInterface+" wrapperIndex = "+wrapperIndex+" wrapperProfile = "+wrapperProfile);
      foundServices = pseInterface.findService(sessionID, serviceInterface, wrapperIndex, wrapperProfile);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in choosing Proxy : "+ex.getMessage());
    }
    if (foundServices == null) {
      throw new PSEException("Error in choosing Proxy : Empty");
    }
    if (foundServices.getNumberOfServices() == 0) {
      throw new PSEException("No matching proxy found");
    }
    bd.setFoundServices(foundServices);
    //FoundServicesProfiles res = new FoundServicesProfiles(foundServices);
    FoundServicesProfiles res = FoundServiceConverter.lower(foundServices);
    return res;
  }

  public int startBridge(int bridgeIndex, int proxyIndex) throws PSEException {
    System.out.println("Sono startBridge di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    FoundServices fs = bd.getFoundServices();
    if (fs == null) {
      throw new PSEException("No proxies found for this service");
    }
    ServiceProxy sp = fs.getService(proxyIndex);
    if (sp == null) {
      throw new PSEException("No proxies present at index "+proxyIndex);
    }
    String brName = bd.getBridgeName();
    if (brName == null) {
      throw new PSEException("No bridge name for this bridge index");
    }
    ServiceBridge sb = null;
    try {
      System.out.println("Cerco di caricare la classe "+brName);
      Class sbClass = Class.forName(brName);
      System.out.println("Ora creo una nuova istanza");
      sb = (ServiceBridge)sbClass.newInstance();
    }
    catch (Exception ex) {
      throw new PSEException("Error in loading bridge class: "+ex);
    }
    if (sb == null) {
      throw new PSEException("No ServiceBridge loaded for this bridge index");
    }
    bd.setServiceBridge(sb);
    int brPort;
    try {
      brPort = sb.start(sp, this);
    }
    catch (PSEException ex) {
      sb.stop();
      throw new PSEException("Error in starting bridge: "+ex.getMessage());
    }
    return brPort;
  }

  public void stopBridge(int bridgeIndex) throws PSEException {
    System.out.println("Sono stopBridge di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    ServiceBridge sb = null;
    sb = bd.getServiceBridge();
    if (sb == null) {
      throw new PSEException("No ServiceBridge loaded for this bridge index");
    }
    sb.stop();
  }

  public void startedService(int bridgeIndex) throws PSEException {
    System.out.println("Sono startedService di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    String si = bd.getServiceInterface();
    if (bd == null) {
      throw new PSEException("No interface specified for service");
    }
    int wi = bd.getWrapperIndex();
    ServiceSessionID sSID;
    /*
    ServiceSessionID sSID = bd.getServiceSessionID();
    if (sSID == null) {
      throw new PSEException("No service session active");
    }
    */
    try {
      sSID = pseInterface.startedService(sessionID, si, wi);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    bd.setServiceSessionID(sSID);
  }

  public void stoppedService(int bridgeIndex) throws PSEException {
    System.out.println("Sono startedService di SurrogateTemp()");
    if (sessionID == null) throw new PSEException("Not logged on");
    //verifico se esite gi quel bridgeindex e se non c' segnalo errore
    Integer brIndex = new Integer(bridgeIndex);
    BridgeDescriptor bd = null;
    if (!bridges.containsKey(brIndex)) {
      throw new PSEException("Bridge not present on surrogate");
    }
    bd = (BridgeDescriptor)bridges.get(brIndex);
    if (bd == null) {
      throw new PSEException("No descritpion associated with this bridge index");
    }
    ServiceSessionID sSID = bd.getServiceSessionID();
    if (sSID == null) {
      throw new PSEException("No service session active");
    }
    try {
      pseInterface.stoppedService(sSID);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  public UserProfileJ2ME getUserProfile() throws PSEException {
    System.out.println("Sono getUserProfile() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    UserProfile up = null;
    try {
      up = pseInterface.getUserProfile(sessionID);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    return ProfileConverter.lower(up);
  }

  public void setUserProfile(UserProfileJ2ME userProfileJ2ME) throws PSEException {
    System.out.println("Sono setUserProfile() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    try {
      pseInterface.setUserProfile(sessionID, ProfileConverter.raise(userProfileJ2ME));
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  public ServicePreferencesJ2ME getServicePreferences(String serviceInterface) throws PSEException {
    System.out.println("Sono getServicePreferences() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    ServicePreferences sp = null;
    try {
      sp = pseInterface.getServicePreferences(sessionID, serviceInterface);
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
    return ProfileConverter.lower(sp);
  }

  public void setServicePreferences(ServicePreferencesJ2ME servicePreferencesJ2ME) throws PSEException {
    System.out.println("Sono setServicePreferences() di SurrogateTemp");
    if (sessionID == null) throw new PSEException("Not logged on");
    try {
      pseInterface.setServicePreferences(sessionID, ProfileConverter.raise(servicePreferencesJ2ME));
    }
    catch (RemoteException ex) {
      throw new PSEException("Error in communicating with server: "+ex.getMessage());
    }
  }

  public ProfileHandler getProfileHandler() throws PSEException {
    try {
      return pseInterface.getProfileHandler();
    } catch (RemoteException e) {
      throw new PSEException("ProfileHandler not found : "+e.getMessage());
    }
  }

  public void abortActivation() {
    System.out.println("Aborting Activation...");
    if (hostContext != null)
      hostContext.cancelActivation();
  }

  class ConnectingThread extends Thread {
    SurrogateMessengerListener listener;
    ConnectingThread(SurrogateMessengerListener list) {
      listener = list;
    }
    public void run() {
      //lettura dei parametri
      try {
        readParams();
        messenger = new SurrogateMessenger(listener, s2, s1);
        deviceConnected = true;
      }
      catch (Exception ex) {
        abortActivation();
      }
    }
    public void readParams() throws Exception {
      System.out.println("Sono in readParams()");
      InetAddress addr = ipInterconnectContext.getAddress();
      byte[] initData = ipInterconnectContext.getInitializationData();
      DataInputStream dis = new DataInputStream(new ByteArrayInputStream(initData));
      try {
        // Leggo le due porte delle socket
        int rp = dis.readUnsignedShort();
        dis.close();
        System.out.println("Ho letto : "+rp+". Ora creo le socket su tale porte all'indirizzo remoto"+addr.getHostAddress());
        s1 = new Socket(addr, rp);
        s2 = new Socket(addr, rp);
        /*
        System.out.println("readParams - 3");
        appDis = new DataInputStream(appSock.getInputStream());
        System.out.println("readParams - 4");
        appDos = new DataOutputStream(appSock.getOutputStream());
        System.out.println("readParams - 5");
        */
      } catch(IOException e) {
        //abortActivation("Failed to activate surrogate: ", e);
        System.out.println("Errore nell'attivazione-lettura del context del surrogate "+e.getMessage());
        throw new Exception();
      }
    }
  }

  public void deviceAlive() {
    //TODO Qua settare la veriabile a true
    deviceAlive = true;
  }

  class KeepAliveHandlerImpl implements KeepAliveHandler {
    public synchronized void keepAlive(long period) throws Exception {
      System.out.println("KeepAliveHandlerImpl.keepAlive("+period+")");
      if (deviceConnected) {
        boolean abort = !deviceAlive;
        deviceAlive = false;
        if (abort) {
          abortActivation();
        }
        else {
          try {
            messenger.keepAlive(period);
          }
          catch (PSEException ex) {
            System.out.println("Error in requesting keepAlive: "+ex.getMessage());
            abortActivation();
          }
        }
      }
    }
  }//KeepAliveHandlerImpl

}