package pse.server.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import java.rmi.RMISecurityManager;
import net.jini.discovery.LookupDiscovery;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceItemFilter;
import net.jini.discovery.DiscoveryManagement;

import pse.shared.*;
import pse.shared.framework.*;
import pse.server.*;

/**
 * Questo modulo si occupa del monitoraggio dei servizi adattabili (e poi anche degli adattatori, successivamente)
 * presenti in rete e della selezione di un servizio per un determinato cliente
 */
public class CacheModule implements ServiceDiscoveryListener {

  /** Il ServiceDiscoveryManager utilizzato per monitorare i servizi nella rete */
  ServiceDiscoveryManager clientMgr = null;
  /** La cache dove i servizi individuati sono mantenuti localmente */
  LookupCache cache = null;
  /** Interfaccia grafica del modulo */
  CacheModuleUI ui;
  /** Filtro per i metodi dell'interfaccia grafica */
  CacheModuleUIFilter uiFilter;

  public CacheModule() throws PSEException {
  }

  /**
   * Comincia il monitoraggio della rete
   * @param mgr Il DiscoveryManagement utilizzato per effettuare il lookup dei servizi
   */
  public void startSearch(DiscoveryManagement mgr) throws PSEException {
    try {
      //LookupDiscoveryManager mgr = new LookupDiscoveryManager(LookupDiscovery.ALL_GROUPS, null /* unicast locators */, null /* DiscoveryListener */);
      clientMgr = new ServiceDiscoveryManager(mgr, new LeaseRenewalManager());
    } catch(Exception e) {
      //e.printStackTrace();
      //System.exit(1);
      System.out.println("Errore in CacheModule "+e.getMessage());
      throw new PSEException("CacheModule error: Coudn't lookup services");
    }

    Class [] classes = new Class[] {ServiceProxy.class};
    ServiceTemplate template = new ServiceTemplate(null, classes, null);

    try {
      cache = clientMgr.createLookupCache(template, null, /* no filter */ this /* no listener */);
    } catch(Exception e) {
      //e.printStackTrace();
      //System.exit(1);
      System.out.println("Errore in CacheModule "+e.getMessage());
      throw new PSEException("CacheModule error: Coudn't lookup services");
    }
  }

  /**************************************/
  /* Metodi di ServiceDiscoveryListener */
  /**************************************/

  public void serviceAdded(ServiceDiscoveryEvent evt) {
      // evt.getPreEventServiceItem() == null
      ServiceItem postItem = evt.getPostEventServiceItem();
      System.out.println("CacheModule: Service appeared: " +
                         postItem.service.getClass().toString());
      uiFilter.serviceAdded((ServiceProxy)postItem.service);
  }

  public void serviceChanged(ServiceDiscoveryEvent evt) {
      ServiceItem preItem = evt.getPostEventServiceItem();
      ServiceItem postItem = evt.getPreEventServiceItem() ;
      System.out.println("CacheModule: Service changed: " +
                         postItem.service.getClass().toString());
      uiFilter.serviceChanged((ServiceProxy)postItem.service, (ServiceProxy)preItem.service);
  }
  public void serviceRemoved(ServiceDiscoveryEvent evt) {
      // evt.getPostEventServiceItem() == null
      ServiceItem preItem = evt.getPreEventServiceItem();
      System.out.println("CacheModule: Service disappeared: " +
                         preItem.service.getClass().toString());
      uiFilter.serviceRemoved((ServiceProxy)preItem.service);
  }

  /********************************/
  /* Metodi di ricerca di servizi */
  /********************************/

  //TODO: per ora come se ci fossero al massimo 10 servizi per una interfaccia. Generalizzare
  //TODO: Naturalmento non viene ancora fatta nessuna ricerca e valutazione su profili, n tantomeno con adattatori
  /**
   * Seleziona i proxy per un servizio adatti ad determinato cliente
   * @param prof profilo completo del cliente relativamente al servizio
   * @serviceInterface interfaccia del servizio
   * @return la lista dei servitori trovati
   */
  FoundServices findServices(SpecificClientProfile prof, String serviceInterface) throws PSEException{
    uiFilter.log("CacheModule: Mi  stato richiesto di scegliere un servizio per l'interfaccia : "+serviceInterface+". Con profilo "+prof);
    //Vector found = new Vector();
    ServiceItem serviceItems [];
    FoundServices found = new FoundServices();
    try {
      serviceItems = cache.lookup(new CustomizableFilter(Class.forName(serviceInterface)), 10);
    }
    catch (ClassNotFoundException ex) {
      throw new PSEException("Interface not present on the server : "+ex.getMessage());
    }
    try {
      System.out.println("Ho trovato "+serviceItems.length+" proxy che corrispondono all'interfaccia. Ora controllo i profili");
      for (int i = 0; i < serviceItems.length; i++) {
        System.out.println("Itero "+i);
        ServiceProxy serv = (ServiceProxy)((ServiceItem)serviceItems[i]).service;
        //if (serv.getProfile().getValue() == prof.getValue()) {
        if (serv!=null) {
          if (match(prof, serv.getProfile())) {
            System.out.println("Il proxy "+i+" va bene");
            found.addService((ServiceProxy)((ServiceItem)serviceItems[i]).service);
          }
          else {
            System.out.println("Il proxy "+i+" non va bene");
          }
        }
      }
    }
    catch (Exception e) {
      throw new PSEException("Error in finding proxies : "+e.getMessage());
    }
    System.out.println("Prima di uiFilter");
    uiFilter.log("CacheModule. I proxy trovati sono: "+found.toString());
    System.out.println("Dopo di uiFilter");
    return found;

    //if (instance == null) throw new PSEException("Proxy not available");
    //System.out.println("Sono chooseService(). Ritorno un oggetto della classe : "+((ServiceItem)instance).service.getClass().getName());
    //return ((ServiceItem)instance).service;
  }

  //TODO. Metodo temporaneo che dovrebbe effettuare un confronto tra i profili utente e server
  //Da generalizzare introducendo i moduli di valutazione dei servizi ed uno generico nel caso di default
  /**
   * Verifica la compatibilit tra il profilo cliente ed il profilo del server
   * @param clp il profilo del client
   * @param sp il profilo del server
   * @return true se sono compatibili
   */
  private boolean match(SpecificClientProfile clp, ServerProfile sp) {
    //TODO (tra l'altro considerare che i profili potrebbero anche essere null)
    System.out.println("Sono in match. Dovrei fare il controllo");
    return true;
  }

  /*******************************************/
  /* Metodi relativi all'interfaccia grafica */
  /*******************************************/

  public void setUI(CacheModuleUI cmui) {
    this.ui = cmui;
    uiFilter = new CacheModuleUIFilter(ui);
    setupUI();
  }

  public void setupUI() {
  }

  //Utile se uso una sola cache per tutti i servizi. Altrimenti, se vado sulla strada
  //di una cache per ogni tipo di servizio non ne ho pi bisogno...
  /**
   * Classe filtro, che verifica se un ServiceItem appartiene o meno ad un ndeterminata classe
   */
  private class CustomizableFilter implements ServiceItemFilter {

    Class cl;

    public CustomizableFilter(Class cl) {
      this.cl = cl;
    }

    public boolean check(ServiceItem item) {
      Object service = item.service;
      //System.out.println("Invocato check() Class = "+cl.getName()+" instance = "+service.getClass().getName());
      if (cl.isInstance(service)) {
        //System.out.println("Il risultato e' true");
        return true;
      }
      else {
        //System.out.println("Il risultato e' false");
        return false;
      }
    }

  }//CustomizableFilter

  /**
   * Classe che filtra le invocazioni sull'interfaccia grafica,
   * mandando messaggi su standard output nel caso non sia stata impostata
   */
  class CacheModuleUIFilter {
    CacheModuleUI cmui;
    CacheModuleUIFilter(CacheModuleUI ui) {
      cmui = ui;
    }
    public void serviceAdded(ServiceProxy serv) {
      if (cmui!=null) {
        cmui.serviceAdded(serv);
      }
    }
    public void serviceChanged(ServiceProxy newVers, ServiceProxy oldVers) {
      if (cmui!=null) {
        cmui.serviceChanged(newVers, oldVers);
      }
    }
    public void serviceRemoved(ServiceProxy serv) {
      if (cmui!=null) {
        cmui.serviceRemoved(serv);
      }
    }
    public void log(String msg) {
      if (cmui!=null) {
        cmui.log(msg);
      }
      else {
        System.out.println(msg+"\n");
      }
    }
  }//CacheModuleUIFilter

}