package pse.server.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;

import java.util.*;

//TODO casomai introdurre una classe per i profili pi adatta; es CompleteUserPreferences
//TODO ora foundUser viene invocato quando viene richiesto un login. In realt sarebbe necessario avere una scansione iniziale dei files con i profili o del sistema di directory e reperirli in quel momento l
//cos anhce per la creazione di usersToSessions le cui entry devono di aftto corrispondeer a quelle dell'interfaccia grafica
 /**
  * Questo modulo in teoria verifica il login dell'utente e ne ritrova il profilo memorizzato in rete
  */
public class LoginModule {

  /** L'interfaccia grafica del modulo */
  private LoginModuleUI ui = null;
  /** Il filtro attraverso cui avvengono le invocazioni sull'interfaccia grafica */
  private LoginModuleUIFilter uiFilter = null;
  /** Per comunicare le variazioni di profilo */
  private SessionModule sessionModule;
  /** Mantiene le corrispondenze tra identificatovo della sessione e nome dell'utente */
  private Map sessionsToUser = Collections.synchronizedMap(new TreeMap()); /*sessionID - userName*/
  /** Mantiene le informazioni (Profilo e sessioni attive in quel momento) per un utente */
  private Map userToInfo= Collections.synchronizedMap(new TreeMap()); /*userName - UserInfo */

  /**
   * Costruttore.
   * @param sm il SessionModule locale con cui  in contatto e che dovr notificare in caso di cambiamenti dei profili
   */
  public LoginModule(SessionModule sm) {
    sessionModule = sm;
  }

  /**
   * Effettua il login per un utente, recuperandone, se ha successo, il profilo mantenuto in rete (UserProfile e ServicePreferences)
   * @param uID nome dell'utente
   * @param pwd password dell'utente
   * @return i componenti del profilo mantenuti in rete assemblati in un ClientProfile
   * @throws PSEException se l'utente non  riconisciuto o la password  errata
   */
  public ClientProfile login(String uID, String pwd) throws PSEException {
    uiFilter.log("LoginModule.login() uID = "+uID+" pwd = "+pwd);
    ClientProfile clp = new ClientProfile("Nome del ClientProfile dato da Login Module");
    UserProfile up = new UserProfile();
    up.addAttribute(new ProfileAttribute("made by","ClientProfile.login"));
    ServicePreferences sp1 = new ServicePreferences("pse.temp.shared.TempInterface");
    sp1.addAttribute(new ProfileAttribute("n1", "v1"));
    sp1.addAttribute(new ProfileAttribute("n2", "v2"));
    ServicePreferences sp2 = new ServicePreferences("pse.temp.shared.SboroneInterface");
    sp2.addAttribute(new ProfileAttribute("n3", "v3"));
    clp.setUserProfile(up);
    clp.setServicePreferences(sp1);
    clp.setServicePreferences(sp2);
    if (!userToInfo.containsKey(uID)) {
      userToInfo.put(uID, new UserInfo(clp));
      uiFilter.foundUser(uID);
    }
    return new ClientProfile(clp);
  }

/* Metodi per la gestione delle sessioni */

  /**
   * Aggiunge una sessione per un utente on-line
   * @param userName nome dell'utente
   * @param sID identificativo della sessione
   */
  synchronized public void addSession(String userName, SessionID sID) {
    sessionsToUser.put(sID, userName);
    if (userToInfo.containsKey(userName)) {
      ((UserInfo)userToInfo.get(userName)).addSession(sID);
      uiFilter.changedInfo(userName);
      uiFilter.log("Added session for "+userName+": "+sID);
    }
    else {
      /*
      Vector sIDs = new Vector();
      sIDs.add(sID);
      userToInfo.put(userName, sIDs);
      */
      //Errore!
    }
  }

  /**
   * Rimuove una sessione per un utente on-line
   * @param sID identificativo della sessione
   */
  synchronized public void removeSession(SessionID sID) {
    if (!sessionsToUser.containsKey(sID)) return; //Errore!
    String userName = (String)sessionsToUser.get(sID);
    if (userName == null) return;
    ((UserInfo)userToInfo.get(userName)).removeSessions(sID);
    uiFilter.changedInfo(userName);
    uiFilter.log("Removed session for "+userName+": "+sID);
  }

/* Metodi per la gestione dei profili */

  /**
   * Ritorna lo UserProfile per un utente connesso
   * @param sID sessione attiva dell'utente
   * @return lo UserProfile per l'utente
   * @throws PSEException se non  stato possibile recuperare il profilo
   */
  public UserProfile getUserProfile(SessionID sID) throws PSEException {
    if (!sessionsToUser.containsKey(sID))
      throw new PSEException("Session "+sID+" not found");
    String userName = (String)sessionsToUser.get(sID);
    if (userName == null)
      throw new PSEException("No userName found for session "+sID);
    ClientProfile cp = ((UserInfo)userToInfo.get(userName)).getClientProfile();
    if (cp==null)
      throw new PSEException("No profile associated with session "+sID);
    uiFilter.log("Requested UserProfile for "+userName);
    return cp.getUserProfile();
  }

  /**
   * Imposta lo UserProfile per un utente
   * @param sID identificativo della sessione dell'utente
   * @param newProf il nuovo UserProfile
   * @throws PSEException se l'operazione non  stata possibile
   */
  public void setUserProfile(SessionID sID, UserProfile newProf) throws PSEException {
    if (!sessionsToUser.containsKey(sID))
      throw new PSEException("Session "+sID+" not found");
    String userName = (String)sessionsToUser.get(sID);
    if (userName == null)
      throw new PSEException("No userName found for session "+sID);
    UserInfo userInfo = (UserInfo)userToInfo.get(userName);
    ClientProfile cp = userInfo.getClientProfile();
    if (cp==null)
      throw new PSEException("No profile associated with session "+sID);
    cp.setUserProfile(newProf);
    //Ora aggiorno tutte le sezioni
    Vector sessions = userInfo.getSessions();
    for(int i=0; i<sessions.size(); i++) {
      sessionModule.setUserProfile((SessionID)sessions.elementAt(i), newProf);
    }
    uiFilter.changedInfo(userName);
    uiFilter.log("Set UserProfile for "+userName+" : "+newProf);
  }

  /**
   * Ritorna le ServicePreferences per un utente connesso ed un servizio
   * @param sID sessione attiva dell'utente
   * @param serviceInterface interfaccia del servizio
   * @return le ServicePreferences per l'utente ed il servizio specificati
   * @throws PSEException se non  stato possibile recuperare il profilo
   */
  public ServicePreferences getServicePreferences(SessionID sID, String serviceInterface) throws PSEException {
    if (!sessionsToUser.containsKey(sID))
      throw new PSEException("Session "+sID+" not found");
    String userName = (String)sessionsToUser.get(sID);
    if (userName == null)
      throw new PSEException("No userName found for session "+sID);
    ClientProfile cp = ((UserInfo)userToInfo.get(userName)).getClientProfile();
    if (cp==null)
      throw new PSEException("No profile associated with session "+sID);
    uiFilter.log("Requested ServicePreferences for "+userName+" and service "+serviceInterface);
    return cp.getServicePreferences(serviceInterface);
  }

  /**
   * Imposta le ServicePreferences di un utente ed un servizio
   * @param sID identificativo della sessione dell'utente
   * @param newProf le nuove ServicePreferences
   * @throws PSEException se l'operazione non  stata possibile
   */
  public void setServicePreferences(SessionID sID, ServicePreferences newProf) throws PSEException {
    if (!sessionsToUser.containsKey(sID))
      throw new PSEException("Session "+sID+" not found");
    String userName = (String)sessionsToUser.get(sID);
    if (userName == null)
      throw new PSEException("No userName found for session "+sID);
    UserInfo userInfo = (UserInfo)userToInfo.get(userName);
    ClientProfile cp = userInfo.getClientProfile();
    if (cp==null)
      throw new PSEException("No profile associated with session "+sID);
    cp.setServicePreferences(newProf);
    //Ora aggiorno tutte le sezioni
    Vector sessions = userInfo.getSessions();
    for(int i=0; i<sessions.size(); i++) {
      sessionModule.setServicePreferences((SessionID)sessions.elementAt(i), newProf);
    }
    uiFilter.log("Set SevicePreference for "+userName+" and service "+newProf.getServiceInterface()+" : "+newProf);
    uiFilter.changedInfo(userName);
  }

/* Metodi per il trattamento dell'interfaccia grafica */

  /**
   * Imposta l'interfaccia grafica del modulo
   * @param ui classe che implementa l'interfaccia grafica del modulo
   */
  public void setUI(LoginModuleUI ui) {
    this.ui = ui;
    uiFilter = new LoginModuleUIFilter(ui);
    setupUI();
  }

  /**
   * Inizializza l'interfaccia grafica
   */
  private void setupUI() {
    if (ui!=null) {
      //TODO
    }
  }

/* Metodi invocati dall'interfaccia grafica */

  /**
   * Restituisce il profilo mantenuto in rete di un utente
   * @param userName nome dell'utente
   * @return il profilo contenente UserProfile e ServicePreferences
   */
  ClientProfile getClientProfile(String userName) {
    if (!userToInfo.containsKey(userName)) return null; //Errore
    return ((UserInfo)userToInfo.get(userName)).getClientProfile();
  }

  /**
   * Ritorna la lista delle sessioni attive per un utente
   * @param userName nome dell'utente
   * @return la lista degli identificativi delle sessioni attive per quell'utente
   */
  Vector getActiveSessions(String userName) {
    if (!userToInfo.containsKey(userName)) return null; //Errore
    return ((UserInfo)userToInfo.get(userName)).getSessions();
  }

  /**
   * Classe che filtra le invocazioni sull'interfaccia grafica,
   * mandando messaggi su standard output nel caso non sia stata impostata
   */
  class LoginModuleUIFilter {
    LoginModuleUI lmui;
    LoginModuleUIFilter (LoginModuleUI lmui) {
      this.lmui  = lmui;
    }
    void log(String msg) {
      if (lmui!=null) {
        lmui.log(msg);
      }
      else {
        System.out.println(msg+"\n");
      }
    }
    void foundUser(String userName) {
      if (lmui!=null) {
        lmui.foundUser(userName);
      }
    }
    void lostUser(String userName) {
      if (lmui!=null) {
        lmui.lostUser(userName);
      }
    }
    void changedInfo(String userName) {
      if (lmui!=null) {
        lmui.changedInfo(userName);
      }
    }
  }//LoginModuleUIFilter

  /**
   * Classe che contiene tutte le informazioni relative ad un utente: Profilo e lista delle sessioni attive
   */
  class UserInfo {
    ClientProfile clientProfile;
    Vector sessions;
    UserInfo(ClientProfile cp) {
      clientProfile = cp;
      sessions = new Vector();
    }
    void setClientProfile(ClientProfile cp) {
      clientProfile = cp;
    }
    ClientProfile getClientProfile() {
      return clientProfile;
    }
    void addSession(SessionID sID) {
      sessions.add(sID);
    }
    void removeSessions(SessionID sID) {
      sessions.remove(sID);
    }
    Vector getSessions() {
      return sessions;
    }
  }//UserInfo
}