package pse.server.framework;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import pse.shared.*;
import pse.shared.framework.*;
import pse.server.*;

import java.util.*;

public class SessionModule {

  /** Mantiene le sessioni attive */
  private Hashtable sessions; /*SessionID - SessionDescriptor*/
  /** Interfaccia grafica del modulo*/
  private SessionModuleUI ui;
  /** Il filtro attraverso cui avvengono le invocazioni sull'interfaccia grafica */
  private SessionModuleUIFilter uiFilter;

  public SessionModule() {
    sessions = new Hashtable();
  }

  /**
   * Crea una una nuova sessione per un utente
   * @param userName il nome dell'utente
   * @param personalProf le informazioni sull'utente (UserProfile e ServicePreferences) contenute in rete
   * @param deviceProf il profilo del dispositivo che l'utente sta utilizzando per la sessione
   */
  public SessionID createNewSession(String userName, ClientProfile personalProf, DeviceProfile deviceProf) throws PSEException {
    uiFilter.log("Invocato SessionModule.createNewSession() per userName = "+userName+" con personalProfile = "+personalProf+" e deviceProfile = "+deviceProf);
    //TODO: Soluzione evidentemente temporanea. Trovare un modo migliore per assegnare il SessionID
    java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("'['HH':'mm':'ss'.'SSS']'");
    String ts = sdf.format(new Date());
    SessionID aSID = new SessionID(userName+ts);
    //Qua dovrei fare una sorta di merge di questi due profili
    ClientProfile mergeProf = personalProf;
    mergeProf.setDeviceProfile(deviceProf);
    sessions.put(aSID, new SessionDescriptor(userName, mergeProf));
    uiFilter.log("Associato il SessionID : "+aSID);
    uiFilter.addedSession(aSID);
    return aSID;
  }

  /**
   * Aggiorna il profilo per una sessione, aggiungendo il profilo di un wrapper
   * @param sID identificativo della sessione
   * @param wp profilo del wrapper
   */
  public void updateProfile(SessionID sID, WrapperProfile wp) throws PSEException {
    uiFilter.log("SessionModule.updateProfile() per SessionID = "+sID+" WrapperProfile = "+wp);
    SessionDescriptor currSess = (SessionDescriptor)sessions.get(sID);
    if (currSess == null) throw new PSEException("Session "+sID+" not found");
    currSess.updateProfile(wp);
    uiFilter.modifiedSessions(sID);
  }

  /**
   * Resituisce il profilo di un utente riguardo ad un servizio e relativamente ad un wrapper
   * @param sID identificativo della sessione dell'utente
   * @param serviceName interfaccia del servizio
   * @param wrapperIndex indice del Wrapper utilizzato per il servizio
   * @return il profilo dell'utente riguardo a quel servizio relativamente a quel wrapper
   */
  public SpecificClientProfile getProfile(SessionID sID, String serviceName, int wrapperIndex) throws PSEException {
    //TODO: per ora lo ritorno tutto. Poi fare scelta pi selettiva
    uiFilter.log("SessionModule.getProfile() per session = "+sID+" serviceName = "+serviceName+" index = "+wrapperIndex);
    SessionDescriptor currSess = (SessionDescriptor)sessions.get(sID);
    if (currSess == null) throw new PSEException("Session "+sID+" not found");
    SpecificClientProfile ret = currSess.getProfile(serviceName, wrapperIndex);
    uiFilter.log("Ritorno : "+ret);
    return ret;
  }

  /**
   * Restituisce il profilo del cliente riguardo ad una sessione di servizio
   * @param sSID identificativo della sessione di servizio
   * @return il profilo di un utente riguardo alla sessione di servizio
   */
  public SpecificClientProfile getProfile(ServiceSessionID sSID) throws PSEException {
    uiFilter.log("SessionModule.getProfile() per sSID = "+sSID);
    SessionID sID = sSID.getSessionID();
    SessionDescriptor currSess = (SessionDescriptor)sessions.get(sID);
    if (currSess == null) throw new PSEException("Session "+sID+" not found");
    SpecificClientProfile ret = currSess.getProfile(sSID.getServiceActivationID());
    uiFilter.log("Ritorno : "+ret);
    return ret;
  }

  //TODO: da eliminare? Se ne occupa il LoginModule, ora
  /**
   * Restituisce il profilo di un utente
   * @param sID identificativo della sessione
   * @return il profilo dell'utente con quella sessione
   */
  public UserProfile getUserProfile(SessionID sID) throws PSEException {
    uiFilter.log("SessionModule.getUserProfile() per sID = "+sID);
    if (!sessions.containsKey(sID)) {
      throw new PSEException("Session "+sID+" not active");
    }
    SessionDescriptor sd = (SessionDescriptor)sessions.get(sID);
    if (sd == null) {
      throw new PSEException("No SessionDescriptor associated with session "+sID);
    }
    ClientProfile cp = sd.getProfile();
    if (cp == null) {
      throw new PSEException("No Profile associated with session "+sID);
    }
    return cp.getUserProfile();
  }

  //TODO: da eliminare? Se ne occupa il LoginModule, ora
  public void setUserProfile(SessionID sID, UserProfile userProfile) throws PSEException {
    uiFilter.log("SessionModule.setUserProfile() per sID = "+sID+" userProfile = "+userProfile);
    if (!sessions.containsKey(sID)) {
      throw new PSEException("Session "+sID+" not active");
    }
    SessionDescriptor sd = (SessionDescriptor)sessions.get(sID);
    if (sd == null) {
      throw new PSEException("No SessionDescriptor associated with session "+sID);
    }
    ClientProfile cp = sd.getProfile();
    if (cp == null) {
      throw new PSEException("No Profile associated with session "+sID);
    }
    cp.setUserProfile(userProfile);
    uiFilter.modifiedSessions(sID);
  }

  //TODO: da eliminare? Se ne occupa il LoginModule, ora
  public void setServicePreferences(SessionID sID, ServicePreferences servicePreferences) throws PSEException {
    uiFilter.log("SessionModule.setServicePreferences() per sID = "+sID+" servicePreferences = "+servicePreferences);
    if (!sessions.containsKey(sID)) {
      throw new PSEException("Session "+sID+" not active");
    }
    SessionDescriptor sd = (SessionDescriptor)sessions.get(sID);
    if (sd == null) {
      throw new PSEException("No SessionDescriptor associated with session "+sID);
    }
    ClientProfile cp = sd.getProfile();
    if (cp == null) {
      throw new PSEException("No Profile associated with session "+sID);
    }
    cp.setServicePreferences(servicePreferences);
    uiFilter.modifiedSessions(sID);
  }

  /**
   * Rimuove una sessione
   * @param identificativo della sessione
   */
  public void removeSession(SessionID sID) throws PSEException {
    uiFilter.log("SessionModule.removeSession() per sID = "+sID);
    //if (!sessions.containsKey(sID)) throws new PSEException("Session not Active");
    sessions.remove(sID);
    uiFilter.removedSession(sID);
  }

  //TODO: Da abbellire
  /**
   * Notifica l'avvio di servizio da parte del cliente
   * @param sID identificativo della sessione
   * @serviceInterface interfaccia del servizio
   * @ident identificativo del wrapper utilizzato
   * @return l'identificativo di sessione di servizio assegnato univocamente
   */
  public ServiceSessionID startedService(SessionID sID, String serviceInterface, int ident) throws PSEException {
    uiFilter.log("SessionModule.startedService() per sID = "+sID+" serviceInterface = "+serviceInterface+" ident = "+ident);
    if (!sessions.containsKey(sID)) throw new PSEException("Session not Active");
    //Creazione del ServiceActivationID, migliore di quella di seguito
    java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("'['HH':'mm':'ss'.'SSS']'");
    String ts = sdf.format(new Date());
    ServiceActivationID sAID = new ServiceActivationID(ts);
    SessionDescriptor sD = (SessionDescriptor)sessions.get(sID);
    //Bisognerebbe controllare che non ci sia gi il doppione di sAID
    sD.serviceActivated(serviceInterface, ident, sAID);
    ServiceSessionID ret = new ServiceSessionID(sID, sAID);
    uiFilter.log("Ritorno : "+ret);
    uiFilter.modifiedSessions(sID);
    return ret;
  }

  //TODO: Da abbellire. Ma poi lanciare errore se non c'?
  /**
   * Notifica la fine di un servizio da parte del cliente
   * @param sSID l'identificativo della sessione di servizio associato
   */
  public void stoppedService(ServiceSessionID sSID) throws PSEException {
    uiFilter.log("SessionModule.stoppedService() per sSID = "+sSID);
    SessionID sID = sSID.getSessionID();
    SessionDescriptor sD = (SessionDescriptor)sessions.get(sID);
    sD.serviceDeactivated(sSID.getServiceActivationID());
    uiFilter.modifiedSessions(sID);
  }

  /********************************************/
  /* Metodi riguardanti l'interfaccia grafica */
  /********************************************/

  public void setUI(SessionModuleUI ui) {
    this.ui = ui;
    uiFilter = new SessionModuleUIFilter(ui);
    setupUI();
  }

  private void setupUI() {
    uiFilter.initSessions(sessions.keys());
  }

  /** Utilizzto dall'interfaccia grafica */
  protected SessionDescriptor getSessionDescriptor(SessionID sID) {
    return (SessionDescriptor)sessions.get(sID);
  }

  class SessionModuleUIFilter {
    SessionModuleUI sui;
    SessionModuleUIFilter(SessionModuleUI sui) {
      this.sui = sui;
    }
    void setUI(SessionModuleUI sui) {
      this.sui = sui;
    }
    public void initSessions(Enumeration en) {
      if (sui!=null) {
        sui.initSessions(en);
      }
    }
    public void addedSession(SessionID sID) {
      if (sui!=null) {
        sui.addedSession(sID);
      }
    }
    public void removedSession(SessionID sID) {
      if (sui!=null) {
        sui.removedSession(sID);
      }
    }
    public void modifiedSessions(SessionID sID) {
      if (sui!=null) {
        sui.modifiedSession(sID);
      }
    }
    public void log(String msg) {
      if (sui!=null) {
        sui.log(msg);
      }
      else System.out.println(msg);
    }
  }//UIFilter

}